from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="FpsEmployeeStarterOccPension")


@attr.s(auto_attribs=True)
class FpsEmployeeStarterOccPension:
    """
    Attributes:
        bereaved (Union[Unset, None, str]):
        amount (Union[Unset, None, str]):
    """

    bereaved: Union[Unset, None, str] = UNSET
    amount: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        bereaved = self.bereaved
        amount = self.amount

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if bereaved is not UNSET:
            field_dict["bereaved"] = bereaved
        if amount is not UNSET:
            field_dict["amount"] = amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bereaved = d.pop("bereaved", UNSET)

        amount = d.pop("amount", UNSET)

        fps_employee_starter_occ_pension = cls(
            bereaved=bereaved,
            amount=amount,
        )

        return fps_employee_starter_occ_pension



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.exb_p11_db_class_1a import ExbP11DbClass1A
from ..types import UNSET, Unset

T = TypeVar("T", bound="ExbP11Db")


@attr.s(auto_attribs=True)
class ExbP11Db:
    """
    Attributes:
        class_1_acontributions_due (Union[Unset, ExbP11DbClass1A]):
    """

    class_1_acontributions_due: Union[Unset, ExbP11DbClass1A] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        class_1_acontributions_due: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.class_1_acontributions_due, Unset):
            class_1_acontributions_due = self.class_1_acontributions_due.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if class_1_acontributions_due is not UNSET:
            field_dict["class1AcontributionsDue"] = class_1_acontributions_due

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _class_1_acontributions_due = d.pop("class1AcontributionsDue", UNSET)
        class_1_acontributions_due: Union[Unset, ExbP11DbClass1A]
        if isinstance(_class_1_acontributions_due, Unset):
            class_1_acontributions_due = UNSET
        else:
            class_1_acontributions_due = ExbP11DbClass1A.from_dict(
                _class_1_acontributions_due
            )

        exb_p11_db = cls(
            class_1_acontributions_due=class_1_acontributions_due,
        )

        return exb_p11_db



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="LoanCharge")


@attr.s(auto_attribs=True)
class LoanCharge:
    """Part of the TaxYearConfig that our engine uses to calculate charges for a Loan.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            threshhold (Union[Unset, float]):
            official_interest_rate (Union[Unset, float]):
    """

    threshhold: Union[Unset, float] = UNSET
    official_interest_rate: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        threshhold = self.threshhold
        official_interest_rate = self.official_interest_rate

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if threshhold is not UNSET:
            field_dict["threshhold"] = threshhold
        if official_interest_rate is not UNSET:
            field_dict["officialInterestRate"] = official_interest_rate

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        threshhold = d.pop("threshhold", UNSET)

        official_interest_rate = d.pop("officialInterestRate", UNSET)

        loan_charge = cls(
            threshhold=threshhold,
            official_interest_rate=official_interest_rate,
        )

        return loan_charge



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Department")


@attr.s(auto_attribs=True)
class Department:
    """
    Attributes:
        code (str): The unique code for this Department
        title (str): The name of this Department
        color (Union[Unset, None, str]): A color to used to represent this Department, in hex format. ie 'ff0000'
        employee_count (Union[Unset, int]): [readonly] The number of employees with this set as their primary department
        accounting_code (Union[Unset, None, str]):
    """

    code: str
    title: str
    color: Union[Unset, None, str] = UNSET
    employee_count: Union[Unset, int] = UNSET
    accounting_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        code = self.code
        title = self.title
        color = self.color
        employee_count = self.employee_count
        accounting_code = self.accounting_code

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "code": code,
                "title": title,
            }
        )
        if color is not UNSET:
            field_dict["color"] = color
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if accounting_code is not UNSET:
            field_dict["accountingCode"] = accounting_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        code = d.pop("code")

        title = d.pop("title")

        color = d.pop("color", UNSET)

        employee_count = d.pop("employeeCount", UNSET)

        accounting_code = d.pop("accountingCode", UNSET)

        department = cls(
            code=code,
            title=title,
            color=color,
            employee_count=employee_count,
            accounting_code=accounting_code,
        )

        return department



from enum import Enum


class BankHolidayCollection(str, Enum):
    NONE = "None"
    ENGLANDANDWALES = "EnglandAndWales"
    SCOTLAND = "Scotland"
    NORTHERNIRELAND = "NorthernIreland"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PayRunStateChangeReason(str, Enum):
    OTHER = "Other"
    ADDITIONALDATANOTSUBMITTED = "AdditionalDataNotSubmitted"
    CHANGETOORIGINALDATA = "ChangeToOriginalData"
    PAYBUREAUERROR = "PayBureauError"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.car_charge_rate import CarChargeRate
from ..types import UNSET, Unset

T = TypeVar("T", bound="CarCharge")


@attr.s(auto_attribs=True)
class CarCharge:
    """Part of the TaxYearConfig that our engine uses to calculate charges for a Company Car.
    It is used internally when our engine performs calculations.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            diesel_surcharge (Union[Unset, float]):
            max_charge (Union[Unset, float]):
            min_charge (Union[Unset, float]):
            fuel_charge (Union[Unset, float]):
            new_car_rate_reduction_date (Union[Unset, None, datetime.date]):
            new_car_rate_reduction_amount (Union[Unset, float]):
            co_2_table (Union[Unset, None, List[CarChargeRate]]):
            engine_size_table (Union[Unset, None, List[CarChargeRate]]):
            zero_emissions_table (Union[Unset, None, List[CarChargeRate]]):
    """

    diesel_surcharge: Union[Unset, float] = UNSET
    max_charge: Union[Unset, float] = UNSET
    min_charge: Union[Unset, float] = UNSET
    fuel_charge: Union[Unset, float] = UNSET
    new_car_rate_reduction_date: Union[Unset, None, datetime.date] = UNSET
    new_car_rate_reduction_amount: Union[Unset, float] = UNSET
    co_2_table: Union[Unset, None, List[CarChargeRate]] = UNSET
    engine_size_table: Union[Unset, None, List[CarChargeRate]] = UNSET
    zero_emissions_table: Union[Unset, None, List[CarChargeRate]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        diesel_surcharge = self.diesel_surcharge
        max_charge = self.max_charge
        min_charge = self.min_charge
        fuel_charge = self.fuel_charge
        new_car_rate_reduction_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.new_car_rate_reduction_date, Unset):
            new_car_rate_reduction_date = (
                self.new_car_rate_reduction_date.isoformat()
                if self.new_car_rate_reduction_date
                else None
            )

        new_car_rate_reduction_amount = self.new_car_rate_reduction_amount
        co_2_table: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.co_2_table, Unset):
            if self.co_2_table is None:
                co_2_table = None
            else:
                co_2_table = []
                for co_2_table_item_data in self.co_2_table:
                    co_2_table_item = co_2_table_item_data.to_dict()

                    co_2_table.append(co_2_table_item)

        engine_size_table: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.engine_size_table, Unset):
            if self.engine_size_table is None:
                engine_size_table = None
            else:
                engine_size_table = []
                for engine_size_table_item_data in self.engine_size_table:
                    engine_size_table_item = engine_size_table_item_data.to_dict()

                    engine_size_table.append(engine_size_table_item)

        zero_emissions_table: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.zero_emissions_table, Unset):
            if self.zero_emissions_table is None:
                zero_emissions_table = None
            else:
                zero_emissions_table = []
                for zero_emissions_table_item_data in self.zero_emissions_table:
                    zero_emissions_table_item = zero_emissions_table_item_data.to_dict()

                    zero_emissions_table.append(zero_emissions_table_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if diesel_surcharge is not UNSET:
            field_dict["dieselSurcharge"] = diesel_surcharge
        if max_charge is not UNSET:
            field_dict["maxCharge"] = max_charge
        if min_charge is not UNSET:
            field_dict["minCharge"] = min_charge
        if fuel_charge is not UNSET:
            field_dict["fuelCharge"] = fuel_charge
        if new_car_rate_reduction_date is not UNSET:
            field_dict["newCarRateReductionDate"] = new_car_rate_reduction_date
        if new_car_rate_reduction_amount is not UNSET:
            field_dict["newCarRateReductionAmount"] = new_car_rate_reduction_amount
        if co_2_table is not UNSET:
            field_dict["co2Table"] = co_2_table
        if engine_size_table is not UNSET:
            field_dict["engineSizeTable"] = engine_size_table
        if zero_emissions_table is not UNSET:
            field_dict["zeroEmissionsTable"] = zero_emissions_table

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        diesel_surcharge = d.pop("dieselSurcharge", UNSET)

        max_charge = d.pop("maxCharge", UNSET)

        min_charge = d.pop("minCharge", UNSET)

        fuel_charge = d.pop("fuelCharge", UNSET)

        _new_car_rate_reduction_date = d.pop("newCarRateReductionDate", UNSET)
        new_car_rate_reduction_date: Union[Unset, None, datetime.date]
        if _new_car_rate_reduction_date is None:
            new_car_rate_reduction_date = None
        elif isinstance(_new_car_rate_reduction_date, Unset):
            new_car_rate_reduction_date = UNSET
        else:
            new_car_rate_reduction_date = isoparse(_new_car_rate_reduction_date).date()

        new_car_rate_reduction_amount = d.pop("newCarRateReductionAmount", UNSET)

        co_2_table = []
        _co_2_table = d.pop("co2Table", UNSET)
        for co_2_table_item_data in _co_2_table or []:
            co_2_table_item = CarChargeRate.from_dict(co_2_table_item_data)

            co_2_table.append(co_2_table_item)

        engine_size_table = []
        _engine_size_table = d.pop("engineSizeTable", UNSET)
        for engine_size_table_item_data in _engine_size_table or []:
            engine_size_table_item = CarChargeRate.from_dict(
                engine_size_table_item_data
            )

            engine_size_table.append(engine_size_table_item)

        zero_emissions_table = []
        _zero_emissions_table = d.pop("zeroEmissionsTable", UNSET)
        for zero_emissions_table_item_data in _zero_emissions_table or []:
            zero_emissions_table_item = CarChargeRate.from_dict(
                zero_emissions_table_item_data
            )

            zero_emissions_table.append(zero_emissions_table_item)

        car_charge = cls(
            diesel_surcharge=diesel_surcharge,
            max_charge=max_charge,
            min_charge=min_charge,
            fuel_charge=fuel_charge,
            new_car_rate_reduction_date=new_car_rate_reduction_date,
            new_car_rate_reduction_amount=new_car_rate_reduction_amount,
            co_2_table=co_2_table,
            engine_size_table=engine_size_table,
            zero_emissions_table=zero_emissions_table,
        )

        return car_charge



from enum import Enum


class BackgroundTaskStatus(str, Enum):
    QUEUED = "Queued"
    SENT = "Sent"
    FAILED = "Failed"
    PROCESSING = "Processing"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AnalysisReportLineValue")


@attr.s(auto_attribs=True)
class AnalysisReportLineValue:
    """
    Attributes:
        qty (Union[Unset, None, float]):
        code (Union[Unset, None, str]):
        value (Union[Unset, float]):
    """

    qty: Union[Unset, None, float] = UNSET
    code: Union[Unset, None, str] = UNSET
    value: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        qty = self.qty
        code = self.code
        value = self.value

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if qty is not UNSET:
            field_dict["qty"] = qty
        if code is not UNSET:
            field_dict["code"] = code
        if value is not UNSET:
            field_dict["value"] = value

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        qty = d.pop("qty", UNSET)

        code = d.pop("code", UNSET)

        value = d.pop("value", UNSET)

        analysis_report_line_value = cls(
            qty=qty,
            code=code,
            value=value,
        )

        return analysis_report_line_value



from enum import Enum


class BenefitDetailsCarPowerType(str, Enum):
    DIESEL6D = "Diesel6d"
    DIESELNON6D = "DieselNon6d"
    PETROL = "Petrol"
    HYBRID = "Hybrid"
    ELECTRIC = "Electric"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DDescOther")


@attr.s(auto_attribs=True)
class P11DDescOther:
    """
    Attributes:
        desc (Union[Unset, None, str]):
        other (Union[Unset, None, str]):
        ann_val_pro_rata (Union[Unset, None, str]):
        gross_or_amt_forgone (Union[Unset, None, str]):
        cost_or_amt_forgone (Union[Unset, None, str]):
        made_good (Union[Unset, None, str]):
        cash_equiv_or_relevant_amt (Union[Unset, None, str]):
    """

    desc: Union[Unset, None, str] = UNSET
    other: Union[Unset, None, str] = UNSET
    ann_val_pro_rata: Union[Unset, None, str] = UNSET
    gross_or_amt_forgone: Union[Unset, None, str] = UNSET
    cost_or_amt_forgone: Union[Unset, None, str] = UNSET
    made_good: Union[Unset, None, str] = UNSET
    cash_equiv_or_relevant_amt: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        desc = self.desc
        other = self.other
        ann_val_pro_rata = self.ann_val_pro_rata
        gross_or_amt_forgone = self.gross_or_amt_forgone
        cost_or_amt_forgone = self.cost_or_amt_forgone
        made_good = self.made_good
        cash_equiv_or_relevant_amt = self.cash_equiv_or_relevant_amt

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if desc is not UNSET:
            field_dict["desc"] = desc
        if other is not UNSET:
            field_dict["other"] = other
        if ann_val_pro_rata is not UNSET:
            field_dict["annValProRata"] = ann_val_pro_rata
        if gross_or_amt_forgone is not UNSET:
            field_dict["grossOrAmtForgone"] = gross_or_amt_forgone
        if cost_or_amt_forgone is not UNSET:
            field_dict["costOrAmtForgone"] = cost_or_amt_forgone
        if made_good is not UNSET:
            field_dict["madeGood"] = made_good
        if cash_equiv_or_relevant_amt is not UNSET:
            field_dict["cashEquivOrRelevantAmt"] = cash_equiv_or_relevant_amt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        desc = d.pop("desc", UNSET)

        other = d.pop("other", UNSET)

        ann_val_pro_rata = d.pop("annValProRata", UNSET)

        gross_or_amt_forgone = d.pop("grossOrAmtForgone", UNSET)

        cost_or_amt_forgone = d.pop("costOrAmtForgone", UNSET)

        made_good = d.pop("madeGood", UNSET)

        cash_equiv_or_relevant_amt = d.pop("cashEquivOrRelevantAmt", UNSET)

        p11d_desc_other = cls(
            desc=desc,
            other=other,
            ann_val_pro_rata=ann_val_pro_rata,
            gross_or_amt_forgone=gross_or_amt_forgone,
            cost_or_amt_forgone=cost_or_amt_forgone,
            made_good=made_good,
            cash_equiv_or_relevant_amt=cash_equiv_or_relevant_amt,
        )

        return p11d_desc_other



import datetime
from typing import Any, Dict, Optional, Union

import httpx

from ...client import Client
from ...models.report_response import ReportResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    all_tenants: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/tenants/{id}/reports/weeklyactivity".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(accept, Unset):
        headers["accept"] = accept

    params: Dict[str, Any] = {}
    json_from_date: Union[Unset, None, str] = UNSET
    if not isinstance(from_date, Unset):
        json_from_date = from_date.isoformat() if from_date else None

    params["fromDate"] = json_from_date

    json_to_date: Union[Unset, None, str] = UNSET
    if not isinstance(to_date, Unset):
        json_to_date = to_date.isoformat() if to_date else None

    params["toDate"] = json_to_date

    params["allTenants"] = all_tenants

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[ReportResponse]:
    if response.status_code == 200:
        response_200 = ReportResponse.from_dict(response.json())

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[ReportResponse]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    all_tenants: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Weekly Activity Report

     Returns a report detailing User activity, summarised by week, for the given date range

    Args:
        id (str):
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        all_tenants (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        from_date=from_date,
        to_date=to_date,
        all_tenants=all_tenants,
        accept=accept,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    all_tenants: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Weekly Activity Report

     Returns a report detailing User activity, summarised by week, for the given date range

    Args:
        id (str):
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        all_tenants (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return sync_detailed(
        id=id,
        client=client,
        from_date=from_date,
        to_date=to_date,
        all_tenants=all_tenants,
        accept=accept,
    ).parsed


async def asyncio_detailed(
    id: str,
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    all_tenants: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Response[ReportResponse]:
    """Weekly Activity Report

     Returns a report detailing User activity, summarised by week, for the given date range

    Args:
        id (str):
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        all_tenants (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        from_date=from_date,
        to_date=to_date,
        all_tenants=all_tenants,
        accept=accept,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    *,
    client: Client,
    from_date: Union[Unset, None, datetime.datetime] = UNSET,
    to_date: Union[Unset, None, datetime.datetime] = UNSET,
    all_tenants: Union[Unset, None, bool] = False,
    accept: Union[Unset, str] = UNSET,
) -> Optional[ReportResponse]:
    """Weekly Activity Report

     Returns a report detailing User activity, summarised by week, for the given date range

    Args:
        id (str):
        from_date (Union[Unset, None, datetime.datetime]):
        to_date (Union[Unset, None, datetime.datetime]):
        all_tenants (Union[Unset, None, bool]):
        accept (Union[Unset, str]):

    Returns:
        Response[ReportResponse]
    """

    return (
        await asyncio_detailed(
            id=id,
            client=client,
            from_date=from_date,
            to_date=to_date,
            all_tenants=all_tenants,
            accept=accept,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DetailedNiValues")


@attr.s(auto_attribs=True)
class P11DetailedNiValues:
    """Forms the NI Summary table in the P11 Detailed report.

    Attributes:
        table (Union[Unset, None, str]): [readonly]
        earnings_at_lel (Union[Unset, float]): [readonly]
        earnings_above_lel_to_pt (Union[Unset, float]): [readonly]
        earnings_above_pt_to_uap (Union[Unset, float]): [readonly]
        earnings_above_uap_to_uel (Union[Unset, float]): [readonly]
        earnings_above_pt_to_uel (Union[Unset, float]): [readonly]
        employee_nics (Union[Unset, float]): [readonly]
        employer_nics (Union[Unset, float]): [readonly]
        employee_and_employer_nics (Union[Unset, float]): [readonly]
    """

    table: Union[Unset, None, str] = UNSET
    earnings_at_lel: Union[Unset, float] = UNSET
    earnings_above_lel_to_pt: Union[Unset, float] = UNSET
    earnings_above_pt_to_uap: Union[Unset, float] = UNSET
    earnings_above_uap_to_uel: Union[Unset, float] = UNSET
    earnings_above_pt_to_uel: Union[Unset, float] = UNSET
    employee_nics: Union[Unset, float] = UNSET
    employer_nics: Union[Unset, float] = UNSET
    employee_and_employer_nics: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        table = self.table
        earnings_at_lel = self.earnings_at_lel
        earnings_above_lel_to_pt = self.earnings_above_lel_to_pt
        earnings_above_pt_to_uap = self.earnings_above_pt_to_uap
        earnings_above_uap_to_uel = self.earnings_above_uap_to_uel
        earnings_above_pt_to_uel = self.earnings_above_pt_to_uel
        employee_nics = self.employee_nics
        employer_nics = self.employer_nics
        employee_and_employer_nics = self.employee_and_employer_nics

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if table is not UNSET:
            field_dict["table"] = table
        if earnings_at_lel is not UNSET:
            field_dict["earningsAtLel"] = earnings_at_lel
        if earnings_above_lel_to_pt is not UNSET:
            field_dict["earningsAboveLelToPt"] = earnings_above_lel_to_pt
        if earnings_above_pt_to_uap is not UNSET:
            field_dict["earningsAbovePtToUap"] = earnings_above_pt_to_uap
        if earnings_above_uap_to_uel is not UNSET:
            field_dict["earningsAboveUapToUel"] = earnings_above_uap_to_uel
        if earnings_above_pt_to_uel is not UNSET:
            field_dict["earningsAbovePtToUel"] = earnings_above_pt_to_uel
        if employee_nics is not UNSET:
            field_dict["employeeNics"] = employee_nics
        if employer_nics is not UNSET:
            field_dict["employerNics"] = employer_nics
        if employee_and_employer_nics is not UNSET:
            field_dict["employeeAndEmployerNics"] = employee_and_employer_nics

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        table = d.pop("table", UNSET)

        earnings_at_lel = d.pop("earningsAtLel", UNSET)

        earnings_above_lel_to_pt = d.pop("earningsAboveLelToPt", UNSET)

        earnings_above_pt_to_uap = d.pop("earningsAbovePtToUap", UNSET)

        earnings_above_uap_to_uel = d.pop("earningsAboveUapToUel", UNSET)

        earnings_above_pt_to_uel = d.pop("earningsAbovePtToUel", UNSET)

        employee_nics = d.pop("employeeNics", UNSET)

        employer_nics = d.pop("employerNics", UNSET)

        employee_and_employer_nics = d.pop("employeeAndEmployerNics", UNSET)

        p11_detailed_ni_values = cls(
            table=table,
            earnings_at_lel=earnings_at_lel,
            earnings_above_lel_to_pt=earnings_above_lel_to_pt,
            earnings_above_pt_to_uap=earnings_above_pt_to_uap,
            earnings_above_uap_to_uel=earnings_above_uap_to_uel,
            earnings_above_pt_to_uel=earnings_above_pt_to_uel,
            employee_nics=employee_nics,
            employer_nics=employer_nics,
            employee_and_employer_nics=employee_and_employer_nics,
        )

        return p11_detailed_ni_values



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="AttachmentOrderPayment")


@attr.s(auto_attribs=True)
class AttachmentOrderPayment:
    """Used to represent a payment towards an Attachment of Earnings Order (AEO)

    Attributes:
        attachment_order (Union[Unset, Item]):
        tax_year (Union[Unset, TaxYear]):
        pay_period (Union[Unset, PayPeriods]):
        date (Union[Unset, datetime.date]): [readonly] The date this deduction was made
        description (Union[Unset, None, str]): [readonly] Description of deduction, usually the reference from the
            AttachmentOrder
        attachable_pay (Union[Unset, float]): [readonly] The total Attachable pay for the Employee on this PayRun
        protected_pay (Union[Unset, float]): [readonly] The amount of the Attachable Pay that must be protected
        other_attachment_order_deductions (Union[Unset, float]): [readonly] Deductions made on this PayRun for this
            employee due to  other higher priority AttachmentOrders
        desired_deduction (Union[Unset, float]): [readonly] The amount that should be deducted, if arrestable pay is
            available.
            Not including any shortfall
        shortfall (Union[Unset, float]): [readonly] Any shortfall (ie arrears) that existed before this PayRun.
        total_paid (Union[Unset, float]): [readonly] Total amount paid to date, including this deduction.
        resulting_deduction (Union[Unset, float]): [readonly] The actual amount deducted
        resulting_shortfall (Union[Unset, float]): [readonly] The resulting shortfall (including any shortfall from
            previous periods
        admin_fee (Union[Unset, float]): [readonly] Any admin fee charged
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    attachment_order: Union[Unset, Item] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    description: Union[Unset, None, str] = UNSET
    attachable_pay: Union[Unset, float] = UNSET
    protected_pay: Union[Unset, float] = UNSET
    other_attachment_order_deductions: Union[Unset, float] = UNSET
    desired_deduction: Union[Unset, float] = UNSET
    shortfall: Union[Unset, float] = UNSET
    total_paid: Union[Unset, float] = UNSET
    resulting_deduction: Union[Unset, float] = UNSET
    resulting_shortfall: Union[Unset, float] = UNSET
    admin_fee: Union[Unset, float] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        attachment_order: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.attachment_order, Unset):
            attachment_order = self.attachment_order.to_dict()

        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        description = self.description
        attachable_pay = self.attachable_pay
        protected_pay = self.protected_pay
        other_attachment_order_deductions = self.other_attachment_order_deductions
        desired_deduction = self.desired_deduction
        shortfall = self.shortfall
        total_paid = self.total_paid
        resulting_deduction = self.resulting_deduction
        resulting_shortfall = self.resulting_shortfall
        admin_fee = self.admin_fee
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if attachment_order is not UNSET:
            field_dict["attachmentOrder"] = attachment_order
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if date is not UNSET:
            field_dict["date"] = date
        if description is not UNSET:
            field_dict["description"] = description
        if attachable_pay is not UNSET:
            field_dict["attachablePay"] = attachable_pay
        if protected_pay is not UNSET:
            field_dict["protectedPay"] = protected_pay
        if other_attachment_order_deductions is not UNSET:
            field_dict[
                "otherAttachmentOrderDeductions"
            ] = other_attachment_order_deductions
        if desired_deduction is not UNSET:
            field_dict["desiredDeduction"] = desired_deduction
        if shortfall is not UNSET:
            field_dict["shortfall"] = shortfall
        if total_paid is not UNSET:
            field_dict["totalPaid"] = total_paid
        if resulting_deduction is not UNSET:
            field_dict["resultingDeduction"] = resulting_deduction
        if resulting_shortfall is not UNSET:
            field_dict["resultingShortfall"] = resulting_shortfall
        if admin_fee is not UNSET:
            field_dict["adminFee"] = admin_fee
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _attachment_order = d.pop("attachmentOrder", UNSET)
        attachment_order: Union[Unset, Item]
        if isinstance(_attachment_order, Unset):
            attachment_order = UNSET
        else:
            attachment_order = Item.from_dict(_attachment_order)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        description = d.pop("description", UNSET)

        attachable_pay = d.pop("attachablePay", UNSET)

        protected_pay = d.pop("protectedPay", UNSET)

        other_attachment_order_deductions = d.pop(
            "otherAttachmentOrderDeductions", UNSET
        )

        desired_deduction = d.pop("desiredDeduction", UNSET)

        shortfall = d.pop("shortfall", UNSET)

        total_paid = d.pop("totalPaid", UNSET)

        resulting_deduction = d.pop("resultingDeduction", UNSET)

        resulting_shortfall = d.pop("resultingShortfall", UNSET)

        admin_fee = d.pop("adminFee", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        attachment_order_payment = cls(
            attachment_order=attachment_order,
            tax_year=tax_year,
            pay_period=pay_period,
            date=date,
            description=description,
            attachable_pay=attachable_pay,
            protected_pay=protected_pay,
            other_attachment_order_deductions=other_attachment_order_deductions,
            desired_deduction=desired_deduction,
            shortfall=shortfall,
            total_paid=total_paid,
            resulting_deduction=resulting_deduction,
            resulting_shortfall=resulting_shortfall,
            admin_fee=admin_fee,
            employee=employee,
            id=id,
        )

        return attachment_order_payment



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EmployerOpeningBalances")


@attr.s(auto_attribs=True)
class EmployerOpeningBalances:
    """
    Attributes:
        smp_recovered (Union[Unset, float]): Value of Statutory Maternity Pay recovered year to date
        spp_recovered (Union[Unset, float]): Value of Statutory Paternity Pay recovered year to date
        sap_recovered (Union[Unset, float]): Value of Statutory Adoption Pay recovered year to date
        sh_pp_recovered (Union[Unset, float]): Value of Shared Parental Pay recovered year to date
        spbp_recovered (Union[Unset, float]): Value of Statutory Parental Bereavment Pay recovered year to date
        nic_compensation_on_smp (Union[Unset, float]): Value of NIC compensation on SMP year to date
        nic_compensation_on_spp (Union[Unset, float]): Value of NIC compensation on Statutory Paternity Pay year to date
        nic_compensation_on_sap (Union[Unset, float]): Value of NIC compensation on Statutory Adoption Pay year to date
        nic_compensation_on_sh_pp (Union[Unset, float]): Value of NIC compensation on Shared Parental Pay year to date
        nic_compensation_on_spbp (Union[Unset, float]): Value of NIC compensation on Statutory Parental Bereavement Pay
            year to date
        cis_deductions_suffered (Union[Unset, float]): Value of CIS deductions suffered year to date
        apprenticeship_levy (Union[Unset, float]): Value of Apprenticeship levy paid year to date
    """

    smp_recovered: Union[Unset, float] = UNSET
    spp_recovered: Union[Unset, float] = UNSET
    sap_recovered: Union[Unset, float] = UNSET
    sh_pp_recovered: Union[Unset, float] = UNSET
    spbp_recovered: Union[Unset, float] = UNSET
    nic_compensation_on_smp: Union[Unset, float] = UNSET
    nic_compensation_on_spp: Union[Unset, float] = UNSET
    nic_compensation_on_sap: Union[Unset, float] = UNSET
    nic_compensation_on_sh_pp: Union[Unset, float] = UNSET
    nic_compensation_on_spbp: Union[Unset, float] = UNSET
    cis_deductions_suffered: Union[Unset, float] = UNSET
    apprenticeship_levy: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        smp_recovered = self.smp_recovered
        spp_recovered = self.spp_recovered
        sap_recovered = self.sap_recovered
        sh_pp_recovered = self.sh_pp_recovered
        spbp_recovered = self.spbp_recovered
        nic_compensation_on_smp = self.nic_compensation_on_smp
        nic_compensation_on_spp = self.nic_compensation_on_spp
        nic_compensation_on_sap = self.nic_compensation_on_sap
        nic_compensation_on_sh_pp = self.nic_compensation_on_sh_pp
        nic_compensation_on_spbp = self.nic_compensation_on_spbp
        cis_deductions_suffered = self.cis_deductions_suffered
        apprenticeship_levy = self.apprenticeship_levy

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if smp_recovered is not UNSET:
            field_dict["smpRecovered"] = smp_recovered
        if spp_recovered is not UNSET:
            field_dict["sppRecovered"] = spp_recovered
        if sap_recovered is not UNSET:
            field_dict["sapRecovered"] = sap_recovered
        if sh_pp_recovered is not UNSET:
            field_dict["shPPRecovered"] = sh_pp_recovered
        if spbp_recovered is not UNSET:
            field_dict["spbpRecovered"] = spbp_recovered
        if nic_compensation_on_smp is not UNSET:
            field_dict["nicCompensationOnSMP"] = nic_compensation_on_smp
        if nic_compensation_on_spp is not UNSET:
            field_dict["nicCompensationOnSPP"] = nic_compensation_on_spp
        if nic_compensation_on_sap is not UNSET:
            field_dict["nicCompensationOnSAP"] = nic_compensation_on_sap
        if nic_compensation_on_sh_pp is not UNSET:
            field_dict["nicCompensationOnShPP"] = nic_compensation_on_sh_pp
        if nic_compensation_on_spbp is not UNSET:
            field_dict["nicCompensationOnSPBP"] = nic_compensation_on_spbp
        if cis_deductions_suffered is not UNSET:
            field_dict["cisDeductionsSuffered"] = cis_deductions_suffered
        if apprenticeship_levy is not UNSET:
            field_dict["apprenticeshipLevy"] = apprenticeship_levy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        smp_recovered = d.pop("smpRecovered", UNSET)

        spp_recovered = d.pop("sppRecovered", UNSET)

        sap_recovered = d.pop("sapRecovered", UNSET)

        sh_pp_recovered = d.pop("shPPRecovered", UNSET)

        spbp_recovered = d.pop("spbpRecovered", UNSET)

        nic_compensation_on_smp = d.pop("nicCompensationOnSMP", UNSET)

        nic_compensation_on_spp = d.pop("nicCompensationOnSPP", UNSET)

        nic_compensation_on_sap = d.pop("nicCompensationOnSAP", UNSET)

        nic_compensation_on_sh_pp = d.pop("nicCompensationOnShPP", UNSET)

        nic_compensation_on_spbp = d.pop("nicCompensationOnSPBP", UNSET)

        cis_deductions_suffered = d.pop("cisDeductionsSuffered", UNSET)

        apprenticeship_levy = d.pop("apprenticeshipLevy", UNSET)

        employer_opening_balances = cls(
            smp_recovered=smp_recovered,
            spp_recovered=spp_recovered,
            sap_recovered=sap_recovered,
            sh_pp_recovered=sh_pp_recovered,
            spbp_recovered=spbp_recovered,
            nic_compensation_on_smp=nic_compensation_on_smp,
            nic_compensation_on_spp=nic_compensation_on_spp,
            nic_compensation_on_sap=nic_compensation_on_sap,
            nic_compensation_on_sh_pp=nic_compensation_on_sh_pp,
            nic_compensation_on_spbp=nic_compensation_on_spbp,
            cis_deductions_suffered=cis_deductions_suffered,
            apprenticeship_levy=apprenticeship_levy,
        )

        return employer_opening_balances



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.p11_ni_values import P11NiValues
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11Line")


@attr.s(auto_attribs=True)
class P11Line:
    """Lines for the P11 Report

    Attributes:
        date (Union[Unset, datetime.date]): [readonly]
        period (Union[Unset, None, str]): [readonly]
        gross_taxable_pay (Union[Unset, float]): [readonly]
        gross_taxable_pay_ytd (Union[Unset, float]): [readonly]
        tax (Union[Unset, float]): [readonly]
        tax_ytd (Union[Unset, float]): [readonly]
        smp_ytd (Union[Unset, float]): [readonly]
        spp_ytd (Union[Unset, float]): [readonly]
        sap_ytd (Union[Unset, float]): [readonly]
        shpp_ytd (Union[Unset, float]): [readonly]
        spbp_ytd (Union[Unset, float]): [readonly]
        student_loan_ytd (Union[Unset, float]): [readonly]
        postgrad_loan_ytd (Union[Unset, float]): [readonly]
        ni_values (Union[Unset, None, List[P11NiValues]]): [readonly]
    """

    date: Union[Unset, datetime.date] = UNSET
    period: Union[Unset, None, str] = UNSET
    gross_taxable_pay: Union[Unset, float] = UNSET
    gross_taxable_pay_ytd: Union[Unset, float] = UNSET
    tax: Union[Unset, float] = UNSET
    tax_ytd: Union[Unset, float] = UNSET
    smp_ytd: Union[Unset, float] = UNSET
    spp_ytd: Union[Unset, float] = UNSET
    sap_ytd: Union[Unset, float] = UNSET
    shpp_ytd: Union[Unset, float] = UNSET
    spbp_ytd: Union[Unset, float] = UNSET
    student_loan_ytd: Union[Unset, float] = UNSET
    postgrad_loan_ytd: Union[Unset, float] = UNSET
    ni_values: Union[Unset, None, List[P11NiValues]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        period = self.period
        gross_taxable_pay = self.gross_taxable_pay
        gross_taxable_pay_ytd = self.gross_taxable_pay_ytd
        tax = self.tax
        tax_ytd = self.tax_ytd
        smp_ytd = self.smp_ytd
        spp_ytd = self.spp_ytd
        sap_ytd = self.sap_ytd
        shpp_ytd = self.shpp_ytd
        spbp_ytd = self.spbp_ytd
        student_loan_ytd = self.student_loan_ytd
        postgrad_loan_ytd = self.postgrad_loan_ytd
        ni_values: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ni_values, Unset):
            if self.ni_values is None:
                ni_values = None
            else:
                ni_values = []
                for ni_values_item_data in self.ni_values:
                    ni_values_item = ni_values_item_data.to_dict()

                    ni_values.append(ni_values_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if period is not UNSET:
            field_dict["period"] = period
        if gross_taxable_pay is not UNSET:
            field_dict["grossTaxablePay"] = gross_taxable_pay
        if gross_taxable_pay_ytd is not UNSET:
            field_dict["grossTaxablePayYTD"] = gross_taxable_pay_ytd
        if tax is not UNSET:
            field_dict["tax"] = tax
        if tax_ytd is not UNSET:
            field_dict["taxYTD"] = tax_ytd
        if smp_ytd is not UNSET:
            field_dict["smpYTD"] = smp_ytd
        if spp_ytd is not UNSET:
            field_dict["sppYTD"] = spp_ytd
        if sap_ytd is not UNSET:
            field_dict["sapYTD"] = sap_ytd
        if shpp_ytd is not UNSET:
            field_dict["shppYTD"] = shpp_ytd
        if spbp_ytd is not UNSET:
            field_dict["spbpYTD"] = spbp_ytd
        if student_loan_ytd is not UNSET:
            field_dict["studentLoanYTD"] = student_loan_ytd
        if postgrad_loan_ytd is not UNSET:
            field_dict["postgradLoanYTD"] = postgrad_loan_ytd
        if ni_values is not UNSET:
            field_dict["niValues"] = ni_values

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        period = d.pop("period", UNSET)

        gross_taxable_pay = d.pop("grossTaxablePay", UNSET)

        gross_taxable_pay_ytd = d.pop("grossTaxablePayYTD", UNSET)

        tax = d.pop("tax", UNSET)

        tax_ytd = d.pop("taxYTD", UNSET)

        smp_ytd = d.pop("smpYTD", UNSET)

        spp_ytd = d.pop("sppYTD", UNSET)

        sap_ytd = d.pop("sapYTD", UNSET)

        shpp_ytd = d.pop("shppYTD", UNSET)

        spbp_ytd = d.pop("spbpYTD", UNSET)

        student_loan_ytd = d.pop("studentLoanYTD", UNSET)

        postgrad_loan_ytd = d.pop("postgradLoanYTD", UNSET)

        ni_values = []
        _ni_values = d.pop("niValues", UNSET)
        for ni_values_item_data in _ni_values or []:
            ni_values_item = P11NiValues.from_dict(ni_values_item_data)

            ni_values.append(ni_values_item)

        p11_line = cls(
            date=date,
            period=period,
            gross_taxable_pay=gross_taxable_pay,
            gross_taxable_pay_ytd=gross_taxable_pay_ytd,
            tax=tax,
            tax_ytd=tax_ytd,
            smp_ytd=smp_ytd,
            spp_ytd=spp_ytd,
            sap_ytd=sap_ytd,
            shpp_ytd=shpp_ytd,
            spbp_ytd=spbp_ytd,
            student_loan_ytd=student_loan_ytd,
            postgrad_loan_ytd=postgrad_loan_ytd,
            ni_values=ni_values,
        )

        return p11_line



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..models.nvr_employee_details import NvrEmployeeDetails
from ..models.nvr_employment import NvrEmployment
from ..types import UNSET, Unset

T = TypeVar("T", bound="NvrEmployee")


@attr.s(auto_attribs=True)
class NvrEmployee:
    """
    Attributes:
        employee_unique_id (Union[Unset, str]):
        item (Union[Unset, Item]):
        employee_details (Union[Unset, NvrEmployeeDetails]):
        employment (Union[Unset, NvrEmployment]):
    """

    employee_unique_id: Union[Unset, str] = UNSET
    item: Union[Unset, Item] = UNSET
    employee_details: Union[Unset, NvrEmployeeDetails] = UNSET
    employment: Union[Unset, NvrEmployment] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_unique_id = self.employee_unique_id
        item: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.item, Unset):
            item = self.item.to_dict()

        employee_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee_details, Unset):
            employee_details = self.employee_details.to_dict()

        employment: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employment, Unset):
            employment = self.employment.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_unique_id is not UNSET:
            field_dict["employeeUniqueId"] = employee_unique_id
        if item is not UNSET:
            field_dict["item"] = item
        if employee_details is not UNSET:
            field_dict["employeeDetails"] = employee_details
        if employment is not UNSET:
            field_dict["employment"] = employment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_unique_id = d.pop("employeeUniqueId", UNSET)

        _item = d.pop("item", UNSET)
        item: Union[Unset, Item]
        if isinstance(_item, Unset):
            item = UNSET
        else:
            item = Item.from_dict(_item)

        _employee_details = d.pop("employeeDetails", UNSET)
        employee_details: Union[Unset, NvrEmployeeDetails]
        if isinstance(_employee_details, Unset):
            employee_details = UNSET
        else:
            employee_details = NvrEmployeeDetails.from_dict(_employee_details)

        _employment = d.pop("employment", UNSET)
        employment: Union[Unset, NvrEmployment]
        if isinstance(_employment, Unset):
            employment = UNSET
        else:
            employment = NvrEmployment.from_dict(_employment)

        nvr_employee = cls(
            employee_unique_id=employee_unique_id,
            item=item,
            employee_details=employee_details,
            employment=employment,
        )

        return nvr_employee



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..models.pay_periods import PayPeriods
from ..models.pay_run import PayRun
from ..models.pay_schedule_period_events_config import PaySchedulePeriodEventsConfig
from ..models.payment_date_type import PaymentDateType
from ..models.tax_year import TaxYear
from ..types import UNSET, Unset

T = TypeVar("T", bound="PaySchedule")


@attr.s(auto_attribs=True)
class PaySchedule:
    """
    Attributes:
        name (Union[Unset, None, str]): A descriptive name for this PaySchedule
        ordinal (Union[Unset, int]): [readonly] Indicates whether this is first, second, third (etc) PaySchedule for
            this PayPeriod.
        tax_year (Union[Unset, TaxYear]):
        pay_period (Union[Unset, PayPeriods]):
        first_period_end_date (Union[Unset, datetime.date]): The last day of the first pay period
        first_payment_date (Union[Unset, datetime.date]): The first payment date
        payment_date_type (Union[Unset, PaymentDateType]):
        period_lengths (Union[Unset, None, List[int]]): Only applicable for PayPeriod of Custom. Defines the length of
            each period
        high_gross_pay (Union[Unset, float]): A gross pay amount considered high for this PaySchedule
        high_net_pay (Union[Unset, float]): A net pay amount considered high for this PaySchedule
        period_events_config (Union[Unset, None, PaySchedulePeriodEventsConfig]): Only applicable if Bureau
            functionality is enabled. Defines the number of days each event occurs before the Payment Date.
        has_open_pay_run_period (Union[Unset, bool]): [readonly] Will be true if the employer currently has an open
            PayRun for this PayPeriod
        last_period_end_date (Union[Unset, None, datetime.date]): [readonly] The end date of the most recent PayRun on
            this schedule
        last_period_number (Union[Unset, None, int]): [readonly] The period number of the most recent PayRun on this
            schedule
        employee_count (Union[Unset, int]): [readonly] The number of employees paid with this PaySchedule
        is_required (Union[Unset, bool]): [readonly] Whether or not this PaySchedule is required for the employer, ie:
            they have employees to be paid on this PaySchedule
        is_configured (Union[Unset, bool]): [readonly] Whether or not this PaySchedule has been configured and is ready
            for use
        is_year_completed (Union[Unset, bool]): [readonly] Returns true if all PayRuns for this PaySchedule in the
            TaxYear have been completed
        year_end_tasks (Union[Unset, None, List[str]]): [readonly] A list of actions that need to be completed for this
            PaySchedule before this TaxYear can be finalised
        pay_runs (Union[Unset, None, List[Item]]): [readonly] Details of PayRuns for this PaySchedule
        current_pay_run (Union[Unset, Item]):
        next_pay_run (Union[Unset, PayRun]): This model is right at the very heart of the software.
            There is a PayRun for each period in which people are paid.
    """

    name: Union[Unset, None, str] = UNSET
    ordinal: Union[Unset, int] = UNSET
    tax_year: Union[Unset, TaxYear] = UNSET
    pay_period: Union[Unset, PayPeriods] = UNSET
    first_period_end_date: Union[Unset, datetime.date] = UNSET
    first_payment_date: Union[Unset, datetime.date] = UNSET
    payment_date_type: Union[Unset, PaymentDateType] = UNSET
    period_lengths: Union[Unset, None, List[int]] = UNSET
    high_gross_pay: Union[Unset, float] = UNSET
    high_net_pay: Union[Unset, float] = UNSET
    period_events_config: Union[Unset, None, PaySchedulePeriodEventsConfig] = UNSET
    has_open_pay_run_period: Union[Unset, bool] = UNSET
    last_period_end_date: Union[Unset, None, datetime.date] = UNSET
    last_period_number: Union[Unset, None, int] = UNSET
    employee_count: Union[Unset, int] = UNSET
    is_required: Union[Unset, bool] = UNSET
    is_configured: Union[Unset, bool] = UNSET
    is_year_completed: Union[Unset, bool] = UNSET
    year_end_tasks: Union[Unset, None, List[str]] = UNSET
    pay_runs: Union[Unset, None, List[Item]] = UNSET
    current_pay_run: Union[Unset, Item] = UNSET
    next_pay_run: Union[Unset, PayRun] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        ordinal = self.ordinal
        tax_year: Union[Unset, str] = UNSET
        if not isinstance(self.tax_year, Unset):
            tax_year = self.tax_year.value

        pay_period: Union[Unset, str] = UNSET
        if not isinstance(self.pay_period, Unset):
            pay_period = self.pay_period.value

        first_period_end_date: Union[Unset, str] = UNSET
        if not isinstance(self.first_period_end_date, Unset):
            first_period_end_date = self.first_period_end_date.isoformat()

        first_payment_date: Union[Unset, str] = UNSET
        if not isinstance(self.first_payment_date, Unset):
            first_payment_date = self.first_payment_date.isoformat()

        payment_date_type: Union[Unset, str] = UNSET
        if not isinstance(self.payment_date_type, Unset):
            payment_date_type = self.payment_date_type.value

        period_lengths: Union[Unset, None, List[int]] = UNSET
        if not isinstance(self.period_lengths, Unset):
            if self.period_lengths is None:
                period_lengths = None
            else:
                period_lengths = self.period_lengths

        high_gross_pay = self.high_gross_pay
        high_net_pay = self.high_net_pay
        period_events_config: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.period_events_config, Unset):
            period_events_config = (
                self.period_events_config.to_dict()
                if self.period_events_config
                else None
            )

        has_open_pay_run_period = self.has_open_pay_run_period
        last_period_end_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_period_end_date, Unset):
            last_period_end_date = (
                self.last_period_end_date.isoformat()
                if self.last_period_end_date
                else None
            )

        last_period_number = self.last_period_number
        employee_count = self.employee_count
        is_required = self.is_required
        is_configured = self.is_configured
        is_year_completed = self.is_year_completed
        year_end_tasks: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.year_end_tasks, Unset):
            if self.year_end_tasks is None:
                year_end_tasks = None
            else:
                year_end_tasks = self.year_end_tasks

        pay_runs: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.pay_runs, Unset):
            if self.pay_runs is None:
                pay_runs = None
            else:
                pay_runs = []
                for pay_runs_item_data in self.pay_runs:
                    pay_runs_item = pay_runs_item_data.to_dict()

                    pay_runs.append(pay_runs_item)

        current_pay_run: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.current_pay_run, Unset):
            current_pay_run = self.current_pay_run.to_dict()

        next_pay_run: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.next_pay_run, Unset):
            next_pay_run = self.next_pay_run.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if ordinal is not UNSET:
            field_dict["ordinal"] = ordinal
        if tax_year is not UNSET:
            field_dict["taxYear"] = tax_year
        if pay_period is not UNSET:
            field_dict["payPeriod"] = pay_period
        if first_period_end_date is not UNSET:
            field_dict["firstPeriodEndDate"] = first_period_end_date
        if first_payment_date is not UNSET:
            field_dict["firstPaymentDate"] = first_payment_date
        if payment_date_type is not UNSET:
            field_dict["paymentDateType"] = payment_date_type
        if period_lengths is not UNSET:
            field_dict["periodLengths"] = period_lengths
        if high_gross_pay is not UNSET:
            field_dict["highGrossPay"] = high_gross_pay
        if high_net_pay is not UNSET:
            field_dict["highNetPay"] = high_net_pay
        if period_events_config is not UNSET:
            field_dict["periodEventsConfig"] = period_events_config
        if has_open_pay_run_period is not UNSET:
            field_dict["hasOpenPayRunPeriod"] = has_open_pay_run_period
        if last_period_end_date is not UNSET:
            field_dict["lastPeriodEndDate"] = last_period_end_date
        if last_period_number is not UNSET:
            field_dict["lastPeriodNumber"] = last_period_number
        if employee_count is not UNSET:
            field_dict["employeeCount"] = employee_count
        if is_required is not UNSET:
            field_dict["isRequired"] = is_required
        if is_configured is not UNSET:
            field_dict["isConfigured"] = is_configured
        if is_year_completed is not UNSET:
            field_dict["isYearCompleted"] = is_year_completed
        if year_end_tasks is not UNSET:
            field_dict["yearEndTasks"] = year_end_tasks
        if pay_runs is not UNSET:
            field_dict["payRuns"] = pay_runs
        if current_pay_run is not UNSET:
            field_dict["currentPayRun"] = current_pay_run
        if next_pay_run is not UNSET:
            field_dict["nextPayRun"] = next_pay_run

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        ordinal = d.pop("ordinal", UNSET)

        _tax_year = d.pop("taxYear", UNSET)
        tax_year: Union[Unset, TaxYear]
        if isinstance(_tax_year, Unset):
            tax_year = UNSET
        else:
            tax_year = TaxYear(_tax_year)

        _pay_period = d.pop("payPeriod", UNSET)
        pay_period: Union[Unset, PayPeriods]
        if isinstance(_pay_period, Unset):
            pay_period = UNSET
        else:
            pay_period = PayPeriods(_pay_period)

        _first_period_end_date = d.pop("firstPeriodEndDate", UNSET)
        first_period_end_date: Union[Unset, datetime.date]
        if isinstance(_first_period_end_date, Unset):
            first_period_end_date = UNSET
        else:
            first_period_end_date = isoparse(_first_period_end_date).date()

        _first_payment_date = d.pop("firstPaymentDate", UNSET)
        first_payment_date: Union[Unset, datetime.date]
        if isinstance(_first_payment_date, Unset):
            first_payment_date = UNSET
        else:
            first_payment_date = isoparse(_first_payment_date).date()

        _payment_date_type = d.pop("paymentDateType", UNSET)
        payment_date_type: Union[Unset, PaymentDateType]
        if isinstance(_payment_date_type, Unset):
            payment_date_type = UNSET
        else:
            payment_date_type = PaymentDateType(_payment_date_type)

        period_lengths = cast(List[int], d.pop("periodLengths", UNSET))

        high_gross_pay = d.pop("highGrossPay", UNSET)

        high_net_pay = d.pop("highNetPay", UNSET)

        _period_events_config = d.pop("periodEventsConfig", UNSET)
        period_events_config: Union[Unset, None, PaySchedulePeriodEventsConfig]
        if _period_events_config is None:
            period_events_config = None
        elif isinstance(_period_events_config, Unset):
            period_events_config = UNSET
        else:
            period_events_config = PaySchedulePeriodEventsConfig.from_dict(
                _period_events_config
            )

        has_open_pay_run_period = d.pop("hasOpenPayRunPeriod", UNSET)

        _last_period_end_date = d.pop("lastPeriodEndDate", UNSET)
        last_period_end_date: Union[Unset, None, datetime.date]
        if _last_period_end_date is None:
            last_period_end_date = None
        elif isinstance(_last_period_end_date, Unset):
            last_period_end_date = UNSET
        else:
            last_period_end_date = isoparse(_last_period_end_date).date()

        last_period_number = d.pop("lastPeriodNumber", UNSET)

        employee_count = d.pop("employeeCount", UNSET)

        is_required = d.pop("isRequired", UNSET)

        is_configured = d.pop("isConfigured", UNSET)

        is_year_completed = d.pop("isYearCompleted", UNSET)

        year_end_tasks = cast(List[str], d.pop("yearEndTasks", UNSET))

        pay_runs = []
        _pay_runs = d.pop("payRuns", UNSET)
        for pay_runs_item_data in _pay_runs or []:
            pay_runs_item = Item.from_dict(pay_runs_item_data)

            pay_runs.append(pay_runs_item)

        _current_pay_run = d.pop("currentPayRun", UNSET)
        current_pay_run: Union[Unset, Item]
        if isinstance(_current_pay_run, Unset):
            current_pay_run = UNSET
        else:
            current_pay_run = Item.from_dict(_current_pay_run)

        _next_pay_run = d.pop("nextPayRun", UNSET)
        next_pay_run: Union[Unset, PayRun]
        if isinstance(_next_pay_run, Unset):
            next_pay_run = UNSET
        else:
            next_pay_run = PayRun.from_dict(_next_pay_run)

        pay_schedule = cls(
            name=name,
            ordinal=ordinal,
            tax_year=tax_year,
            pay_period=pay_period,
            first_period_end_date=first_period_end_date,
            first_payment_date=first_payment_date,
            payment_date_type=payment_date_type,
            period_lengths=period_lengths,
            high_gross_pay=high_gross_pay,
            high_net_pay=high_net_pay,
            period_events_config=period_events_config,
            has_open_pay_run_period=has_open_pay_run_period,
            last_period_end_date=last_period_end_date,
            last_period_number=last_period_number,
            employee_count=employee_count,
            is_required=is_required,
            is_configured=is_configured,
            is_year_completed=is_year_completed,
            year_end_tasks=year_end_tasks,
            pay_runs=pay_runs,
            current_pay_run=current_pay_run,
            next_pay_run=next_pay_run,
        )

        return pay_schedule



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.deferal_period_type import DeferalPeriodType
from ..models.pension_selection import PensionSelection
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutoEnrolmentSettings")


@attr.s(auto_attribs=True)
class AutoEnrolmentSettings:
    """
    Attributes:
        staging_date (Union[Unset, datetime.date]):
        cyclical_reenrolment_date (Union[Unset, None, datetime.date]):
        previous_cyclical_reenrolment_date (Union[Unset, None, datetime.date]):
        default_pension (Union[Unset, PensionSelection]):
        pension_same_as_default (Union[Unset, bool]):
        days_to_defer_assessment (Union[Unset, int]): The number of days, if any, to defer assessment of new employees.
            You're allowed to defer assessment of new employees by up to 3 months.
            This is the default value used when you create a new employee. It can be changed on a per-employee basis.
        defer_by_months_not_days (Union[Unset, bool]): If set to true then the value in DaysToDeferAssessment will be
            treated as a number of months, not a number of days
        defer_enrolment_by (Union[Unset, int]): The number of days )or weeks, or months), if any, to defer enrolment of
            employees that are Eligible Jobholders.
            For example, if this is set to 30 days then if an employee meets the criteria for enrolment then they'll only be
            enrolled if they still meet the criteria 30 days later
        defer_enrolment_by_period_type (Union[Unset, DeferalPeriodType]):
        include_non_pensioned_employees_in_submission (Union[Unset, bool]): Whether or not to include details of non-
            pensioned employees in your submissions to this provider
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    staging_date: Union[Unset, datetime.date] = UNSET
    cyclical_reenrolment_date: Union[Unset, None, datetime.date] = UNSET
    previous_cyclical_reenrolment_date: Union[Unset, None, datetime.date] = UNSET
    default_pension: Union[Unset, PensionSelection] = UNSET
    pension_same_as_default: Union[Unset, bool] = UNSET
    days_to_defer_assessment: Union[Unset, int] = UNSET
    defer_by_months_not_days: Union[Unset, bool] = UNSET
    defer_enrolment_by: Union[Unset, int] = UNSET
    defer_enrolment_by_period_type: Union[Unset, DeferalPeriodType] = UNSET
    include_non_pensioned_employees_in_submission: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        staging_date: Union[Unset, str] = UNSET
        if not isinstance(self.staging_date, Unset):
            staging_date = self.staging_date.isoformat()

        cyclical_reenrolment_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.cyclical_reenrolment_date, Unset):
            cyclical_reenrolment_date = (
                self.cyclical_reenrolment_date.isoformat()
                if self.cyclical_reenrolment_date
                else None
            )

        previous_cyclical_reenrolment_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.previous_cyclical_reenrolment_date, Unset):
            previous_cyclical_reenrolment_date = (
                self.previous_cyclical_reenrolment_date.isoformat()
                if self.previous_cyclical_reenrolment_date
                else None
            )

        default_pension: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.default_pension, Unset):
            default_pension = self.default_pension.to_dict()

        pension_same_as_default = self.pension_same_as_default
        days_to_defer_assessment = self.days_to_defer_assessment
        defer_by_months_not_days = self.defer_by_months_not_days
        defer_enrolment_by = self.defer_enrolment_by
        defer_enrolment_by_period_type: Union[Unset, str] = UNSET
        if not isinstance(self.defer_enrolment_by_period_type, Unset):
            defer_enrolment_by_period_type = self.defer_enrolment_by_period_type.value

        include_non_pensioned_employees_in_submission = (
            self.include_non_pensioned_employees_in_submission
        )
        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if staging_date is not UNSET:
            field_dict["stagingDate"] = staging_date
        if cyclical_reenrolment_date is not UNSET:
            field_dict["cyclicalReenrolmentDate"] = cyclical_reenrolment_date
        if previous_cyclical_reenrolment_date is not UNSET:
            field_dict[
                "previousCyclicalReenrolmentDate"
            ] = previous_cyclical_reenrolment_date
        if default_pension is not UNSET:
            field_dict["defaultPension"] = default_pension
        if pension_same_as_default is not UNSET:
            field_dict["pensionSameAsDefault"] = pension_same_as_default
        if days_to_defer_assessment is not UNSET:
            field_dict["daysToDeferAssessment"] = days_to_defer_assessment
        if defer_by_months_not_days is not UNSET:
            field_dict["deferByMonthsNotDays"] = defer_by_months_not_days
        if defer_enrolment_by is not UNSET:
            field_dict["deferEnrolmentBy"] = defer_enrolment_by
        if defer_enrolment_by_period_type is not UNSET:
            field_dict["deferEnrolmentByPeriodType"] = defer_enrolment_by_period_type
        if include_non_pensioned_employees_in_submission is not UNSET:
            field_dict[
                "includeNonPensionedEmployeesInSubmission"
            ] = include_non_pensioned_employees_in_submission
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _staging_date = d.pop("stagingDate", UNSET)
        staging_date: Union[Unset, datetime.date]
        if isinstance(_staging_date, Unset):
            staging_date = UNSET
        else:
            staging_date = isoparse(_staging_date).date()

        _cyclical_reenrolment_date = d.pop("cyclicalReenrolmentDate", UNSET)
        cyclical_reenrolment_date: Union[Unset, None, datetime.date]
        if _cyclical_reenrolment_date is None:
            cyclical_reenrolment_date = None
        elif isinstance(_cyclical_reenrolment_date, Unset):
            cyclical_reenrolment_date = UNSET
        else:
            cyclical_reenrolment_date = isoparse(_cyclical_reenrolment_date).date()

        _previous_cyclical_reenrolment_date = d.pop(
            "previousCyclicalReenrolmentDate", UNSET
        )
        previous_cyclical_reenrolment_date: Union[Unset, None, datetime.date]
        if _previous_cyclical_reenrolment_date is None:
            previous_cyclical_reenrolment_date = None
        elif isinstance(_previous_cyclical_reenrolment_date, Unset):
            previous_cyclical_reenrolment_date = UNSET
        else:
            previous_cyclical_reenrolment_date = isoparse(
                _previous_cyclical_reenrolment_date
            ).date()

        _default_pension = d.pop("defaultPension", UNSET)
        default_pension: Union[Unset, PensionSelection]
        if isinstance(_default_pension, Unset):
            default_pension = UNSET
        else:
            default_pension = PensionSelection.from_dict(_default_pension)

        pension_same_as_default = d.pop("pensionSameAsDefault", UNSET)

        days_to_defer_assessment = d.pop("daysToDeferAssessment", UNSET)

        defer_by_months_not_days = d.pop("deferByMonthsNotDays", UNSET)

        defer_enrolment_by = d.pop("deferEnrolmentBy", UNSET)

        _defer_enrolment_by_period_type = d.pop("deferEnrolmentByPeriodType", UNSET)
        defer_enrolment_by_period_type: Union[Unset, DeferalPeriodType]
        if isinstance(_defer_enrolment_by_period_type, Unset):
            defer_enrolment_by_period_type = UNSET
        else:
            defer_enrolment_by_period_type = DeferalPeriodType(
                _defer_enrolment_by_period_type
            )

        include_non_pensioned_employees_in_submission = d.pop(
            "includeNonPensionedEmployeesInSubmission", UNSET
        )

        id = d.pop("id", UNSET)

        auto_enrolment_settings = cls(
            staging_date=staging_date,
            cyclical_reenrolment_date=cyclical_reenrolment_date,
            previous_cyclical_reenrolment_date=previous_cyclical_reenrolment_date,
            default_pension=default_pension,
            pension_same_as_default=pension_same_as_default,
            days_to_defer_assessment=days_to_defer_assessment,
            defer_by_months_not_days=defer_by_months_not_days,
            defer_enrolment_by=defer_enrolment_by,
            defer_enrolment_by_period_type=defer_enrolment_by_period_type,
            include_non_pensioned_employees_in_submission=include_non_pensioned_employees_in_submission,
            id=id,
        )

        return auto_enrolment_settings



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.external_data_provider_id import ExternalDataProviderId
from ..models.item import Item
from ..models.leave_pay_type import LeavePayType
from ..models.leave_type import LeaveType
from ..models.linked_piw import LinkedPiw
from ..types import UNSET, Unset

T = TypeVar("T", bound="Leave")


@attr.s(auto_attribs=True)
class Leave:
    """Used to represent Leave, including Holiday and Statutory leave (such as Maternity Leave)

    Attributes:
        provider_id (Union[Unset, ExternalDataProviderId]):
        external_id (Union[Unset, None, str]): If the Leave comes from an ExternalDataProvider, then this is its Id in
            the ExternalDataProvider
        type (Union[Unset, LeaveType]):
        pay (Union[Unset, LeavePayType]):
        from_ (Union[Unset, datetime.datetime]): The first day of Leave.
            If it's a half day PM then set the time portion to 12:00:00, otherwise leave it blank or set it to 00:00:00
        to (Union[Unset, datetime.datetime]): The last day of Leave.
            If it's a half day AM then set the time portion to 11:59:59, otherwise set it to 23:59:59
        notes (Union[Unset, None, str]): A free-form text field to record any comments
        average_weekly_earnings (Union[Unset, float]): The employees average weekly earnings. Only relevant for
            Statutory Pay
            It's advised that you don't try to calculate this yourself.
        automatic_awe_calculation (Union[Unset, bool]): If set to True then we'll automatically calculate the
            AverageWeeklyEarnings.
            Set it to false if you want to manually provide a figure that overrides our calculations
        baby_date (Union[Unset, None, datetime.date]): Only required for Parental Leave with Statutory Pay
            If Type is Maternity or Paternity then this is the date the baby is due.
            For Adoption this is the Matching Date.
        secondary_baby_date (Union[Unset, None, datetime.date]): Only used for Parental Leave with Statutory Pay
            If Type is Maternity, Paternity, SharedParental (Birth) then this is the the Baby Born Date.
            For Adoption or SharedParental (Adoption) this is the Expected Placement Date.
        tertiary_baby_date (Union[Unset, None, datetime.date]): Only used for Parental Leave with Statutory Pay
            If Type is Adoption this is the Placement Date.
        override_payment_description (Union[Unset, bool]): If Pay is StatutoryPay and you want to override our
            description that goes with the payment then set this to true
        overriden_payment_description (Union[Unset, None, str]): If OverridePaymentDescription is true and Pay is set to
            StatutoryPay then we'll use this as the description for the payment amount.
        working_days (Union[Unset, float]): [readonly] The number of working days covered by this leave.
            This is calculated based on the employees Working Pattern.
        working_days_override (Union[Unset, None, float]): If a value is provided here then this will be used in place
            of the calculated WorkingDays value
        total_days (Union[Unset, float]): [readonly] The number of days covered by this leave, regardless of whether or
            not they're working days.
            This is calculated based on the employees Working Pattern.
        total_days_override (Union[Unset, None, float]): If a value is provided here then this will be used in place of
            the calculated TotalDays value
        use_assumed_pensionable_pay (Union[Unset, bool]): If this Leave has Statutory Pay (and isn't for Sick) then if
            this is set to True
            we will use the value set in AssumedPensionablePay to work out the employer pension contributions
        assumed_pensionable_pay (Union[Unset, None, float]): if AssumedPensionablePay is True, then this is the value
            used to calculate the employer pension contributions
        offset_pay (Union[Unset, bool]): If this Leave has Statutory Pay  and this is set to True and the employe eis
            paid a fixed amoutn per period
            with Leave Adjustments set to automatic, then we'll reduce their pay for the period by the statutory amount
            so the employee still gets paid the full amount.
        ssp_pay_from_day_one (Union[Unset, bool]): If this is Sick Leave with Statutory Pay then setting this to true
            will force SSP to be paid from day one rather than the usual rule
            of the first Working Day after 3 Qualifying Days
        linked_piw (Union[Unset, LinkedPiw]): Linked Period of Incapacity for Work.
            If you record Sick Leave and select Statutory Pay then any other Sick Leave with Statutory Pay
            lasting 4 or more days in the previous 8 weeks will be linked to it
        kit_split_days (Union[Unset, None, List[datetime.datetime]]): If the LeaveType supports KIT/SPLIT days then use
            this property to store the list of dates
        document_count (Union[Unset, int]): [readonly] The number of attachments associated with this model
        documents (Union[Unset, None, List[Item]]): [readonly] The attachments associated with this model
        employee (Union[Unset, Item]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    provider_id: Union[Unset, ExternalDataProviderId] = UNSET
    external_id: Union[Unset, None, str] = UNSET
    type: Union[Unset, LeaveType] = UNSET
    pay: Union[Unset, LeavePayType] = UNSET
    from_: Union[Unset, datetime.datetime] = UNSET
    to: Union[Unset, datetime.datetime] = UNSET
    notes: Union[Unset, None, str] = UNSET
    average_weekly_earnings: Union[Unset, float] = UNSET
    automatic_awe_calculation: Union[Unset, bool] = UNSET
    baby_date: Union[Unset, None, datetime.date] = UNSET
    secondary_baby_date: Union[Unset, None, datetime.date] = UNSET
    tertiary_baby_date: Union[Unset, None, datetime.date] = UNSET
    override_payment_description: Union[Unset, bool] = UNSET
    overriden_payment_description: Union[Unset, None, str] = UNSET
    working_days: Union[Unset, float] = UNSET
    working_days_override: Union[Unset, None, float] = UNSET
    total_days: Union[Unset, float] = UNSET
    total_days_override: Union[Unset, None, float] = UNSET
    use_assumed_pensionable_pay: Union[Unset, bool] = UNSET
    assumed_pensionable_pay: Union[Unset, None, float] = UNSET
    offset_pay: Union[Unset, bool] = UNSET
    ssp_pay_from_day_one: Union[Unset, bool] = UNSET
    linked_piw: Union[Unset, LinkedPiw] = UNSET
    kit_split_days: Union[Unset, None, List[datetime.datetime]] = UNSET
    document_count: Union[Unset, int] = UNSET
    documents: Union[Unset, None, List[Item]] = UNSET
    employee: Union[Unset, Item] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        provider_id: Union[Unset, str] = UNSET
        if not isinstance(self.provider_id, Unset):
            provider_id = self.provider_id.value

        external_id = self.external_id
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        pay: Union[Unset, str] = UNSET
        if not isinstance(self.pay, Unset):
            pay = self.pay.value

        from_: Union[Unset, str] = UNSET
        if not isinstance(self.from_, Unset):
            from_ = self.from_.isoformat()

        to: Union[Unset, str] = UNSET
        if not isinstance(self.to, Unset):
            to = self.to.isoformat()

        notes = self.notes
        average_weekly_earnings = self.average_weekly_earnings
        automatic_awe_calculation = self.automatic_awe_calculation
        baby_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.baby_date, Unset):
            baby_date = self.baby_date.isoformat() if self.baby_date else None

        secondary_baby_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.secondary_baby_date, Unset):
            secondary_baby_date = (
                self.secondary_baby_date.isoformat()
                if self.secondary_baby_date
                else None
            )

        tertiary_baby_date: Union[Unset, None, str] = UNSET
        if not isinstance(self.tertiary_baby_date, Unset):
            tertiary_baby_date = (
                self.tertiary_baby_date.isoformat() if self.tertiary_baby_date else None
            )

        override_payment_description = self.override_payment_description
        overriden_payment_description = self.overriden_payment_description
        working_days = self.working_days
        working_days_override = self.working_days_override
        total_days = self.total_days
        total_days_override = self.total_days_override
        use_assumed_pensionable_pay = self.use_assumed_pensionable_pay
        assumed_pensionable_pay = self.assumed_pensionable_pay
        offset_pay = self.offset_pay
        ssp_pay_from_day_one = self.ssp_pay_from_day_one
        linked_piw: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.linked_piw, Unset):
            linked_piw = self.linked_piw.to_dict()

        kit_split_days: Union[Unset, None, List[str]] = UNSET
        if not isinstance(self.kit_split_days, Unset):
            if self.kit_split_days is None:
                kit_split_days = None
            else:
                kit_split_days = []
                for kit_split_days_item_data in self.kit_split_days:
                    kit_split_days_item = kit_split_days_item_data.isoformat()

                    kit_split_days.append(kit_split_days_item)

        document_count = self.document_count
        documents: Union[Unset, None, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.documents, Unset):
            if self.documents is None:
                documents = None
            else:
                documents = []
                for documents_item_data in self.documents:
                    documents_item = documents_item_data.to_dict()

                    documents.append(documents_item)

        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if provider_id is not UNSET:
            field_dict["providerId"] = provider_id
        if external_id is not UNSET:
            field_dict["externalId"] = external_id
        if type is not UNSET:
            field_dict["type"] = type
        if pay is not UNSET:
            field_dict["pay"] = pay
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to
        if notes is not UNSET:
            field_dict["notes"] = notes
        if average_weekly_earnings is not UNSET:
            field_dict["averageWeeklyEarnings"] = average_weekly_earnings
        if automatic_awe_calculation is not UNSET:
            field_dict["automaticAWECalculation"] = automatic_awe_calculation
        if baby_date is not UNSET:
            field_dict["babyDate"] = baby_date
        if secondary_baby_date is not UNSET:
            field_dict["secondaryBabyDate"] = secondary_baby_date
        if tertiary_baby_date is not UNSET:
            field_dict["tertiaryBabyDate"] = tertiary_baby_date
        if override_payment_description is not UNSET:
            field_dict["overridePaymentDescription"] = override_payment_description
        if overriden_payment_description is not UNSET:
            field_dict["overridenPaymentDescription"] = overriden_payment_description
        if working_days is not UNSET:
            field_dict["workingDays"] = working_days
        if working_days_override is not UNSET:
            field_dict["workingDaysOverride"] = working_days_override
        if total_days is not UNSET:
            field_dict["totalDays"] = total_days
        if total_days_override is not UNSET:
            field_dict["totalDaysOverride"] = total_days_override
        if use_assumed_pensionable_pay is not UNSET:
            field_dict["useAssumedPensionablePay"] = use_assumed_pensionable_pay
        if assumed_pensionable_pay is not UNSET:
            field_dict["assumedPensionablePay"] = assumed_pensionable_pay
        if offset_pay is not UNSET:
            field_dict["offsetPay"] = offset_pay
        if ssp_pay_from_day_one is not UNSET:
            field_dict["sspPayFromDayOne"] = ssp_pay_from_day_one
        if linked_piw is not UNSET:
            field_dict["linkedPiw"] = linked_piw
        if kit_split_days is not UNSET:
            field_dict["kitSplitDays"] = kit_split_days
        if document_count is not UNSET:
            field_dict["documentCount"] = document_count
        if documents is not UNSET:
            field_dict["documents"] = documents
        if employee is not UNSET:
            field_dict["employee"] = employee
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _provider_id = d.pop("providerId", UNSET)
        provider_id: Union[Unset, ExternalDataProviderId]
        if isinstance(_provider_id, Unset):
            provider_id = UNSET
        else:
            provider_id = ExternalDataProviderId(_provider_id)

        external_id = d.pop("externalId", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, LeaveType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = LeaveType(_type)

        _pay = d.pop("pay", UNSET)
        pay: Union[Unset, LeavePayType]
        if isinstance(_pay, Unset):
            pay = UNSET
        else:
            pay = LeavePayType(_pay)

        _from_ = d.pop("from", UNSET)
        from_: Union[Unset, datetime.datetime]
        if isinstance(_from_, Unset):
            from_ = UNSET
        else:
            from_ = isoparse(_from_)

        _to = d.pop("to", UNSET)
        to: Union[Unset, datetime.datetime]
        if isinstance(_to, Unset):
            to = UNSET
        else:
            to = isoparse(_to)

        notes = d.pop("notes", UNSET)

        average_weekly_earnings = d.pop("averageWeeklyEarnings", UNSET)

        automatic_awe_calculation = d.pop("automaticAWECalculation", UNSET)

        _baby_date = d.pop("babyDate", UNSET)
        baby_date: Union[Unset, None, datetime.date]
        if _baby_date is None:
            baby_date = None
        elif isinstance(_baby_date, Unset):
            baby_date = UNSET
        else:
            baby_date = isoparse(_baby_date).date()

        _secondary_baby_date = d.pop("secondaryBabyDate", UNSET)
        secondary_baby_date: Union[Unset, None, datetime.date]
        if _secondary_baby_date is None:
            secondary_baby_date = None
        elif isinstance(_secondary_baby_date, Unset):
            secondary_baby_date = UNSET
        else:
            secondary_baby_date = isoparse(_secondary_baby_date).date()

        _tertiary_baby_date = d.pop("tertiaryBabyDate", UNSET)
        tertiary_baby_date: Union[Unset, None, datetime.date]
        if _tertiary_baby_date is None:
            tertiary_baby_date = None
        elif isinstance(_tertiary_baby_date, Unset):
            tertiary_baby_date = UNSET
        else:
            tertiary_baby_date = isoparse(_tertiary_baby_date).date()

        override_payment_description = d.pop("overridePaymentDescription", UNSET)

        overriden_payment_description = d.pop("overridenPaymentDescription", UNSET)

        working_days = d.pop("workingDays", UNSET)

        working_days_override = d.pop("workingDaysOverride", UNSET)

        total_days = d.pop("totalDays", UNSET)

        total_days_override = d.pop("totalDaysOverride", UNSET)

        use_assumed_pensionable_pay = d.pop("useAssumedPensionablePay", UNSET)

        assumed_pensionable_pay = d.pop("assumedPensionablePay", UNSET)

        offset_pay = d.pop("offsetPay", UNSET)

        ssp_pay_from_day_one = d.pop("sspPayFromDayOne", UNSET)

        _linked_piw = d.pop("linkedPiw", UNSET)
        linked_piw: Union[Unset, LinkedPiw]
        if isinstance(_linked_piw, Unset):
            linked_piw = UNSET
        else:
            linked_piw = LinkedPiw.from_dict(_linked_piw)

        kit_split_days = []
        _kit_split_days = d.pop("kitSplitDays", UNSET)
        for kit_split_days_item_data in _kit_split_days or []:
            kit_split_days_item = isoparse(kit_split_days_item_data)

            kit_split_days.append(kit_split_days_item)

        document_count = d.pop("documentCount", UNSET)

        documents = []
        _documents = d.pop("documents", UNSET)
        for documents_item_data in _documents or []:
            documents_item = Item.from_dict(documents_item_data)

            documents.append(documents_item)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        id = d.pop("id", UNSET)

        leave = cls(
            provider_id=provider_id,
            external_id=external_id,
            type=type,
            pay=pay,
            from_=from_,
            to=to,
            notes=notes,
            average_weekly_earnings=average_weekly_earnings,
            automatic_awe_calculation=automatic_awe_calculation,
            baby_date=baby_date,
            secondary_baby_date=secondary_baby_date,
            tertiary_baby_date=tertiary_baby_date,
            override_payment_description=override_payment_description,
            overriden_payment_description=overriden_payment_description,
            working_days=working_days,
            working_days_override=working_days_override,
            total_days=total_days,
            total_days_override=total_days_override,
            use_assumed_pensionable_pay=use_assumed_pensionable_pay,
            assumed_pensionable_pay=assumed_pensionable_pay,
            offset_pay=offset_pay,
            ssp_pay_from_day_one=ssp_pay_from_day_one,
            linked_piw=linked_piw,
            kit_split_days=kit_split_days,
            document_count=document_count,
            documents=documents,
            employee=employee,
            id=id,
        )

        return leave



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TaxBracket")


@attr.s(auto_attribs=True)
class TaxBracket:
    """
    Attributes:
        from_ (Union[Unset, float]): [readonly] The starting point for applying this tax rate
        to (Union[Unset, float]): [readonly] The end point for applying this tax rate
        multiplier (Union[Unset, float]): [readonly] The tax rate to apply
    """

    from_: Union[Unset, float] = UNSET
    to: Union[Unset, float] = UNSET
    multiplier: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        from_ = self.from_
        to = self.to
        multiplier = self.multiplier

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to
        if multiplier is not UNSET:
            field_dict["multiplier"] = multiplier

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        from_ = d.pop("from", UNSET)

        to = d.pop("to", UNSET)

        multiplier = d.pop("multiplier", UNSET)

        tax_bracket = cls(
            from_=from_,
            to=to,
            multiplier=multiplier,
        )

        return tax_bracket



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="UmbrellaPayment")


@attr.s(auto_attribs=True)
class UmbrellaPayment:
    """
    Attributes:
        payroll_code (Union[Unset, None, str]): When importing multiple UmbrellaPayments this field is used to identify
            the employee
        charge_per_timesheet (Union[Unset, None, float]): Override the settings for this employer by specifying a
            ChargePerTimeSheet.
            Or leave it as null to use the settings from the Employer
        invoice_value (Union[Unset, float]):
        maps_miles (Union[Unset, int]):
        other_expenses (Union[Unset, float]):
        number_of_timesheets (Union[Unset, int]):
        hours_worked (Union[Unset, float]):
        gross_deduction (Union[Unset, float]):
        gross_addition (Union[Unset, float]):
    """

    payroll_code: Union[Unset, None, str] = UNSET
    charge_per_timesheet: Union[Unset, None, float] = UNSET
    invoice_value: Union[Unset, float] = UNSET
    maps_miles: Union[Unset, int] = UNSET
    other_expenses: Union[Unset, float] = UNSET
    number_of_timesheets: Union[Unset, int] = UNSET
    hours_worked: Union[Unset, float] = UNSET
    gross_deduction: Union[Unset, float] = UNSET
    gross_addition: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        payroll_code = self.payroll_code
        charge_per_timesheet = self.charge_per_timesheet
        invoice_value = self.invoice_value
        maps_miles = self.maps_miles
        other_expenses = self.other_expenses
        number_of_timesheets = self.number_of_timesheets
        hours_worked = self.hours_worked
        gross_deduction = self.gross_deduction
        gross_addition = self.gross_addition

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if charge_per_timesheet is not UNSET:
            field_dict["chargePerTimesheet"] = charge_per_timesheet
        if invoice_value is not UNSET:
            field_dict["invoiceValue"] = invoice_value
        if maps_miles is not UNSET:
            field_dict["mapsMiles"] = maps_miles
        if other_expenses is not UNSET:
            field_dict["otherExpenses"] = other_expenses
        if number_of_timesheets is not UNSET:
            field_dict["numberOfTimesheets"] = number_of_timesheets
        if hours_worked is not UNSET:
            field_dict["hoursWorked"] = hours_worked
        if gross_deduction is not UNSET:
            field_dict["grossDeduction"] = gross_deduction
        if gross_addition is not UNSET:
            field_dict["grossAddition"] = gross_addition

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        payroll_code = d.pop("payrollCode", UNSET)

        charge_per_timesheet = d.pop("chargePerTimesheet", UNSET)

        invoice_value = d.pop("invoiceValue", UNSET)

        maps_miles = d.pop("mapsMiles", UNSET)

        other_expenses = d.pop("otherExpenses", UNSET)

        number_of_timesheets = d.pop("numberOfTimesheets", UNSET)

        hours_worked = d.pop("hoursWorked", UNSET)

        gross_deduction = d.pop("grossDeduction", UNSET)

        gross_addition = d.pop("grossAddition", UNSET)

        umbrella_payment = cls(
            payroll_code=payroll_code,
            charge_per_timesheet=charge_per_timesheet,
            invoice_value=invoice_value,
            maps_miles=maps_miles,
            other_expenses=other_expenses,
            number_of_timesheets=number_of_timesheets,
            hours_worked=hours_worked,
            gross_deduction=gross_deduction,
            gross_addition=gross_addition,
        )

        return umbrella_payment



from enum import Enum


class AutoPilotFinaliseTime(str, Enum):
    JUSTAFTERMIDNIGHT = "JustAfterMidnight"
    NINEAM = "NineAm"
    ONEPM = "OnePm"
    FOURPM = "FourPm"
    SIXPM = "SixPm"
    ELEVENPM = "ElevenPm"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CisContractor")


@attr.s(auto_attribs=True)
class CisContractor:
    """Used to represent details of a CIS SubContractor when communicating with the HMRC Gateway

    Attributes:
        utr (Union[Unset, None, str]):
        a_oref (Union[Unset, None, str]):
    """

    utr: Union[Unset, None, str] = UNSET
    a_oref: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        utr = self.utr
        a_oref = self.a_oref

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if utr is not UNSET:
            field_dict["utr"] = utr
        if a_oref is not UNSET:
            field_dict["aOref"] = a_oref

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        utr = d.pop("utr", UNSET)

        a_oref = d.pop("aOref", UNSET)

        cis_contractor = cls(
            utr=utr,
            a_oref=a_oref,
        )

        return cis_contractor



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="HmrcPayment")


@attr.s(auto_attribs=True)
class HmrcPayment:
    """
    Attributes:
        amount (Union[Unset, float]):
        date (Union[Unset, datetime.date]):
        id (Union[Unset, str]): [readonly] The unique id of the object
    """

    amount: Union[Unset, float] = UNSET
    date: Union[Unset, datetime.date] = UNSET
    id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        amount = self.amount
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if amount is not UNSET:
            field_dict["amount"] = amount
        if date is not UNSET:
            field_dict["date"] = date
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        amount = d.pop("amount", UNSET)

        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        id = d.pop("id", UNSET)

        hmrc_payment = cls(
            amount=amount,
            date=date,
            id=id,
        )

        return hmrc_payment



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.average_weekly_earnings_result import AverageWeeklyEarningsResult
from ..types import UNSET, Unset

T = TypeVar("T", bound="AverageWeeklyEarnings")


@attr.s(auto_attribs=True)
class AverageWeeklyEarnings:
    """
    Attributes:
        result (Union[Unset, AverageWeeklyEarningsResult]):
        result_description (Union[Unset, None, str]):
        average_earnings (Union[Unset, float]):
        threshold (Union[Unset, float]):
        eligibility_threshold (Union[Unset, float]):
        requested_date (Union[Unset, datetime.date]):
        relevant_period_start (Union[Unset, datetime.date]):
        relevant_period_end (Union[Unset, datetime.date]):
        relevant_period_week_count (Union[Unset, float]):
        relevant_period_earnings (Union[Unset, float]):
    """

    result: Union[Unset, AverageWeeklyEarningsResult] = UNSET
    result_description: Union[Unset, None, str] = UNSET
    average_earnings: Union[Unset, float] = UNSET
    threshold: Union[Unset, float] = UNSET
    eligibility_threshold: Union[Unset, float] = UNSET
    requested_date: Union[Unset, datetime.date] = UNSET
    relevant_period_start: Union[Unset, datetime.date] = UNSET
    relevant_period_end: Union[Unset, datetime.date] = UNSET
    relevant_period_week_count: Union[Unset, float] = UNSET
    relevant_period_earnings: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        result: Union[Unset, str] = UNSET
        if not isinstance(self.result, Unset):
            result = self.result.value

        result_description = self.result_description
        average_earnings = self.average_earnings
        threshold = self.threshold
        eligibility_threshold = self.eligibility_threshold
        requested_date: Union[Unset, str] = UNSET
        if not isinstance(self.requested_date, Unset):
            requested_date = self.requested_date.isoformat()

        relevant_period_start: Union[Unset, str] = UNSET
        if not isinstance(self.relevant_period_start, Unset):
            relevant_period_start = self.relevant_period_start.isoformat()

        relevant_period_end: Union[Unset, str] = UNSET
        if not isinstance(self.relevant_period_end, Unset):
            relevant_period_end = self.relevant_period_end.isoformat()

        relevant_period_week_count = self.relevant_period_week_count
        relevant_period_earnings = self.relevant_period_earnings

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if result is not UNSET:
            field_dict["result"] = result
        if result_description is not UNSET:
            field_dict["resultDescription"] = result_description
        if average_earnings is not UNSET:
            field_dict["averageEarnings"] = average_earnings
        if threshold is not UNSET:
            field_dict["threshold"] = threshold
        if eligibility_threshold is not UNSET:
            field_dict["eligibilityThreshold"] = eligibility_threshold
        if requested_date is not UNSET:
            field_dict["requestedDate"] = requested_date
        if relevant_period_start is not UNSET:
            field_dict["relevantPeriodStart"] = relevant_period_start
        if relevant_period_end is not UNSET:
            field_dict["relevantPeriodEnd"] = relevant_period_end
        if relevant_period_week_count is not UNSET:
            field_dict["relevantPeriodWeekCount"] = relevant_period_week_count
        if relevant_period_earnings is not UNSET:
            field_dict["relevantPeriodEarnings"] = relevant_period_earnings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _result = d.pop("result", UNSET)
        result: Union[Unset, AverageWeeklyEarningsResult]
        if isinstance(_result, Unset):
            result = UNSET
        else:
            result = AverageWeeklyEarningsResult(_result)

        result_description = d.pop("resultDescription", UNSET)

        average_earnings = d.pop("averageEarnings", UNSET)

        threshold = d.pop("threshold", UNSET)

        eligibility_threshold = d.pop("eligibilityThreshold", UNSET)

        _requested_date = d.pop("requestedDate", UNSET)
        requested_date: Union[Unset, datetime.date]
        if isinstance(_requested_date, Unset):
            requested_date = UNSET
        else:
            requested_date = isoparse(_requested_date).date()

        _relevant_period_start = d.pop("relevantPeriodStart", UNSET)
        relevant_period_start: Union[Unset, datetime.date]
        if isinstance(_relevant_period_start, Unset):
            relevant_period_start = UNSET
        else:
            relevant_period_start = isoparse(_relevant_period_start).date()

        _relevant_period_end = d.pop("relevantPeriodEnd", UNSET)
        relevant_period_end: Union[Unset, datetime.date]
        if isinstance(_relevant_period_end, Unset):
            relevant_period_end = UNSET
        else:
            relevant_period_end = isoparse(_relevant_period_end).date()

        relevant_period_week_count = d.pop("relevantPeriodWeekCount", UNSET)

        relevant_period_earnings = d.pop("relevantPeriodEarnings", UNSET)

        average_weekly_earnings = cls(
            result=result,
            result_description=result_description,
            average_earnings=average_earnings,
            threshold=threshold,
            eligibility_threshold=eligibility_threshold,
            requested_date=requested_date,
            relevant_period_start=relevant_period_start,
            relevant_period_end=relevant_period_end,
            relevant_period_week_count=relevant_period_week_count,
            relevant_period_earnings=relevant_period_earnings,
        )

        return average_weekly_earnings



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TaxCodeChange")


@attr.s(auto_attribs=True)
class TaxCodeChange:
    """Part of the TaxYearConfig that our engine uses to perform the Year End routine.
    It is used internally when our engine performs the Year End routine.
    You do not need to do anything with this model, it's provided purely for informational purposes.

        Attributes:
            suffix (Union[Unset, None, str]): [readonly] The suffix of the Tax Code that needs to be incremented for this
                Tax Year
            increment (Union[Unset, int]): [readonly] The amount by which to increment Tax Codes with the given suffix
    """

    suffix: Union[Unset, None, str] = UNSET
    increment: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        suffix = self.suffix
        increment = self.increment

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if suffix is not UNSET:
            field_dict["suffix"] = suffix
        if increment is not UNSET:
            field_dict["increment"] = increment

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        suffix = d.pop("suffix", UNSET)

        increment = d.pop("increment", UNSET)

        tax_code_change = cls(
            suffix=suffix,
            increment=increment,
        )

        return tax_code_change



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11PayeSummary")


@attr.s(auto_attribs=True)
class P11PayeSummary:
    """Forms the PAYE summary in the P11 Detailed report

    Attributes:
        previous_employment_pay (Union[Unset, float]): [readonly]
        previous_employment_tax (Union[Unset, float]): [readonly]
        this_employment_pay (Union[Unset, float]): [readonly]
        this_employment_tax (Union[Unset, float]): [readonly]
        total_pay_for_year (Union[Unset, float]): [readonly]
        total_tax_for_year (Union[Unset, float]): [readonly]
    """

    previous_employment_pay: Union[Unset, float] = UNSET
    previous_employment_tax: Union[Unset, float] = UNSET
    this_employment_pay: Union[Unset, float] = UNSET
    this_employment_tax: Union[Unset, float] = UNSET
    total_pay_for_year: Union[Unset, float] = UNSET
    total_tax_for_year: Union[Unset, float] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        previous_employment_pay = self.previous_employment_pay
        previous_employment_tax = self.previous_employment_tax
        this_employment_pay = self.this_employment_pay
        this_employment_tax = self.this_employment_tax
        total_pay_for_year = self.total_pay_for_year
        total_tax_for_year = self.total_tax_for_year

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if previous_employment_pay is not UNSET:
            field_dict["previousEmploymentPay"] = previous_employment_pay
        if previous_employment_tax is not UNSET:
            field_dict["previousEmploymentTax"] = previous_employment_tax
        if this_employment_pay is not UNSET:
            field_dict["thisEmploymentPay"] = this_employment_pay
        if this_employment_tax is not UNSET:
            field_dict["thisEmploymentTax"] = this_employment_tax
        if total_pay_for_year is not UNSET:
            field_dict["totalPayForYear"] = total_pay_for_year
        if total_tax_for_year is not UNSET:
            field_dict["totalTaxForYear"] = total_tax_for_year

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        previous_employment_pay = d.pop("previousEmploymentPay", UNSET)

        previous_employment_tax = d.pop("previousEmploymentTax", UNSET)

        this_employment_pay = d.pop("thisEmploymentPay", UNSET)

        this_employment_tax = d.pop("thisEmploymentTax", UNSET)

        total_pay_for_year = d.pop("totalPayForYear", UNSET)

        total_tax_for_year = d.pop("totalTaxForYear", UNSET)

        p11_paye_summary = cls(
            previous_employment_pay=previous_employment_pay,
            previous_employment_tax=previous_employment_tax,
            this_employment_pay=this_employment_pay,
            this_employment_tax=this_employment_tax,
            total_pay_for_year=total_pay_for_year,
            total_tax_for_year=total_tax_for_year,
        )

        return p11_paye_summary



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.p11_detailed_ni_values import P11DetailedNiValues
from ..types import UNSET, Unset

T = TypeVar("T", bound="P11NiAndStatPaymentsLine")


@attr.s(auto_attribs=True)
class P11NiAndStatPaymentsLine:
    """Lines for the NI Contributions and Statutory Payments table in the P11 Detailed Report

    Attributes:
        date (Union[Unset, datetime.date]): [readonly]
        period (Union[Unset, None, str]): [readonly]
        month_number (Union[Unset, None, str]): [readonly]
        week_number (Union[Unset, None, str]): [readonly]
        smp (Union[Unset, float]): [readonly]
        spp (Union[Unset, float]): [readonly]
        sap (Union[Unset, float]): [readonly]
        shpp (Union[Unset, float]): [readonly]
        ssp (Union[Unset, float]): [readonly]
        spbp (Union[Unset, float]): [readonly]
        class1a (Union[Unset, float]): [readonly]
        ni_values (Union[Unset, P11DetailedNiValues]): Forms the NI Summary table in the P11 Detailed report.
        tax_code (Union[Unset, None, str]): [readonly]
    """

    date: Union[Unset, datetime.date] = UNSET
    period: Union[Unset, None, str] = UNSET
    month_number: Union[Unset, None, str] = UNSET
    week_number: Union[Unset, None, str] = UNSET
    smp: Union[Unset, float] = UNSET
    spp: Union[Unset, float] = UNSET
    sap: Union[Unset, float] = UNSET
    shpp: Union[Unset, float] = UNSET
    ssp: Union[Unset, float] = UNSET
    spbp: Union[Unset, float] = UNSET
    class1a: Union[Unset, float] = UNSET
    ni_values: Union[Unset, P11DetailedNiValues] = UNSET
    tax_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date: Union[Unset, str] = UNSET
        if not isinstance(self.date, Unset):
            date = self.date.isoformat()

        period = self.period
        month_number = self.month_number
        week_number = self.week_number
        smp = self.smp
        spp = self.spp
        sap = self.sap
        shpp = self.shpp
        ssp = self.ssp
        spbp = self.spbp
        class1a = self.class1a
        ni_values: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ni_values, Unset):
            ni_values = self.ni_values.to_dict()

        tax_code = self.tax_code

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if period is not UNSET:
            field_dict["period"] = period
        if month_number is not UNSET:
            field_dict["monthNumber"] = month_number
        if week_number is not UNSET:
            field_dict["weekNumber"] = week_number
        if smp is not UNSET:
            field_dict["smp"] = smp
        if spp is not UNSET:
            field_dict["spp"] = spp
        if sap is not UNSET:
            field_dict["sap"] = sap
        if shpp is not UNSET:
            field_dict["shpp"] = shpp
        if ssp is not UNSET:
            field_dict["ssp"] = ssp
        if spbp is not UNSET:
            field_dict["spbp"] = spbp
        if class1a is not UNSET:
            field_dict["class1a"] = class1a
        if ni_values is not UNSET:
            field_dict["niValues"] = ni_values
        if tax_code is not UNSET:
            field_dict["taxCode"] = tax_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _date = d.pop("date", UNSET)
        date: Union[Unset, datetime.date]
        if isinstance(_date, Unset):
            date = UNSET
        else:
            date = isoparse(_date).date()

        period = d.pop("period", UNSET)

        month_number = d.pop("monthNumber", UNSET)

        week_number = d.pop("weekNumber", UNSET)

        smp = d.pop("smp", UNSET)

        spp = d.pop("spp", UNSET)

        sap = d.pop("sap", UNSET)

        shpp = d.pop("shpp", UNSET)

        ssp = d.pop("ssp", UNSET)

        spbp = d.pop("spbp", UNSET)

        class1a = d.pop("class1a", UNSET)

        _ni_values = d.pop("niValues", UNSET)
        ni_values: Union[Unset, P11DetailedNiValues]
        if isinstance(_ni_values, Unset):
            ni_values = UNSET
        else:
            ni_values = P11DetailedNiValues.from_dict(_ni_values)

        tax_code = d.pop("taxCode", UNSET)

        p11_ni_and_stat_payments_line = cls(
            date=date,
            period=period,
            month_number=month_number,
            week_number=week_number,
            smp=smp,
            spp=spp,
            sap=sap,
            shpp=shpp,
            ssp=ssp,
            spbp=spbp,
            class1a=class1a,
            ni_values=ni_values,
            tax_code=tax_code,
        )

        return p11_ni_and_stat_payments_line



from enum import Enum


class CalendarEntryType(str, Enum):
    EMPLOYEESTARTING = "EmployeeStarting"
    EMPLOYEELEAVING = "EmployeeLeaving"
    EMPLOYEEABSENCE = "EmployeeAbsence"
    PAYDAY = "PayDay"
    EMPLOYEEBIRTHDAY = "EmployeeBirthday"
    EMPLOYEEWORKANNIVERSARY = "EmployeeWorkAnniversary"

    def __str__(self) -> str:
        return str(self.value)

    @classmethod
    def _missing_(cls, value):
        #  Staffology API sometimes sends the index value inside an enum instead of the value of the enum itself
        value_from_index = list(dict(cls.__members__).values())[int(value)]
        return cls(value_from_index)



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="P11DLoan")


@attr.s(auto_attribs=True)
class P11DLoan:
    """
    Attributes:
        joint (Union[Unset, None, str]):
        init_os (Union[Unset, None, str]):
        final_os (Union[Unset, None, str]):
        max_os (Union[Unset, None, str]):
        int_paid (Union[Unset, None, str]):
        date (Union[Unset, None, str]):
        discharge (Union[Unset, None, str]):
        cash_equiv_or_relevant_amt (Union[Unset, None, str]):
    """

    joint: Union[Unset, None, str] = UNSET
    init_os: Union[Unset, None, str] = UNSET
    final_os: Union[Unset, None, str] = UNSET
    max_os: Union[Unset, None, str] = UNSET
    int_paid: Union[Unset, None, str] = UNSET
    date: Union[Unset, None, str] = UNSET
    discharge: Union[Unset, None, str] = UNSET
    cash_equiv_or_relevant_amt: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        joint = self.joint
        init_os = self.init_os
        final_os = self.final_os
        max_os = self.max_os
        int_paid = self.int_paid
        date = self.date
        discharge = self.discharge
        cash_equiv_or_relevant_amt = self.cash_equiv_or_relevant_amt

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if joint is not UNSET:
            field_dict["joint"] = joint
        if init_os is not UNSET:
            field_dict["initOS"] = init_os
        if final_os is not UNSET:
            field_dict["finalOS"] = final_os
        if max_os is not UNSET:
            field_dict["maxOS"] = max_os
        if int_paid is not UNSET:
            field_dict["intPaid"] = int_paid
        if date is not UNSET:
            field_dict["date"] = date
        if discharge is not UNSET:
            field_dict["discharge"] = discharge
        if cash_equiv_or_relevant_amt is not UNSET:
            field_dict["cashEquivOrRelevantAmt"] = cash_equiv_or_relevant_amt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        joint = d.pop("joint", UNSET)

        init_os = d.pop("initOS", UNSET)

        final_os = d.pop("finalOS", UNSET)

        max_os = d.pop("maxOS", UNSET)

        int_paid = d.pop("intPaid", UNSET)

        date = d.pop("date", UNSET)

        discharge = d.pop("discharge", UNSET)

        cash_equiv_or_relevant_amt = d.pop("cashEquivOrRelevantAmt", UNSET)

        p11d_loan = cls(
            joint=joint,
            init_os=init_os,
            final_os=final_os,
            max_os=max_os,
            int_paid=int_paid,
            date=date,
            discharge=discharge,
            cash_equiv_or_relevant_amt=cash_equiv_or_relevant_amt,
        )

        return p11d_loan



from enum import Enum


class TeachersPensionEmploymentType(str, Enum):
    FULLTIME = "FullTime"
    PARTTIMEREGULAR = "PartTimeRegular"
    IRREGULARPARTTIME = "IrregularPartTime"
    IRREGULARPARTTIME_IN = "IrregularPartTime_In"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.item import Item
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}/{periodNumber}/leave".format(
        client.base_url,
        employerId=employer_id,
        taxYear=tax_year,
        payPeriod=pay_period,
        periodNumber=period_number,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[List[Item]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = Item.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[List[Item]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[Item]]:
    """List Leave related to a PayRun

     Returns a List of Items representing any Leave within the PayRun date range for Employees that are
    included in the given PayRun.
    Note that if Leave was added after the PayRun was closed, but the Leave is within the PayRun period
    then it'll be returned here even though it wouldn't have been processed with the PayRun.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[Item]]:
    """List Leave related to a PayRun

     Returns a List of Items representing any Leave within the PayRun date range for Employees that are
    included in the given PayRun.
    Note that if Leave was added after the PayRun was closed, but the Leave is within the PayRun period
    then it'll be returned here even though it wouldn't have been processed with the PayRun.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[List[Item]]:
    """List Leave related to a PayRun

     Returns a List of Items representing any Leave within the PayRun date range for Employees that are
    included in the given PayRun.
    Note that if Leave was added after the PayRun was closed, but the Leave is within the PayRun period
    then it'll be returned here even though it wouldn't have been processed with the PayRun.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[List[Item]]:
    """List Leave related to a PayRun

     Returns a List of Items representing any Leave within the PayRun date range for Employees that are
    included in the given PayRun.
    Note that if Leave was added after the PayRun was closed, but the Leave is within the PayRun period
    then it'll be returned here even though it wouldn't have been processed with the PayRun.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.

    Returns:
        Response[List[Item]]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            period_number=period_number,
            client=client,
            ordinal=ordinal,
        )
    ).parsed



import datetime
from typing import Any, Dict, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="FurloughReportLine")


@attr.s(auto_attribs=True)
class FurloughReportLine:
    """
    Attributes:
        employee (Union[Unset, Item]):
        employees_full_name (Union[Unset, None, str]):
        employees_nino (Union[Unset, None, str]):
        employees_payroll_reference_number (Union[Unset, None, str]):
        furloughed_days (Union[Unset, int]):
        days_in_period (Union[Unset, int]):
        hours_normally_worked (Union[Unset, None, float]):
        hours_furloughed (Union[Unset, None, float]):
        percentage_of_furloughed_days_worked (Union[Unset, float]):
        gross_pay (Union[Unset, float]):
        gross_pay_claim (Union[Unset, float]):
        ni_claim (Union[Unset, float]):
        pension_claim (Union[Unset, float]):
        total_claim (Union[Unset, float]):
        furlough_start (Union[Unset, None, datetime.date]):
        furlough_end (Union[Unset, None, datetime.date]):
        department (Union[Unset, None, str]):
    """

    employee: Union[Unset, Item] = UNSET
    employees_full_name: Union[Unset, None, str] = UNSET
    employees_nino: Union[Unset, None, str] = UNSET
    employees_payroll_reference_number: Union[Unset, None, str] = UNSET
    furloughed_days: Union[Unset, int] = UNSET
    days_in_period: Union[Unset, int] = UNSET
    hours_normally_worked: Union[Unset, None, float] = UNSET
    hours_furloughed: Union[Unset, None, float] = UNSET
    percentage_of_furloughed_days_worked: Union[Unset, float] = UNSET
    gross_pay: Union[Unset, float] = UNSET
    gross_pay_claim: Union[Unset, float] = UNSET
    ni_claim: Union[Unset, float] = UNSET
    pension_claim: Union[Unset, float] = UNSET
    total_claim: Union[Unset, float] = UNSET
    furlough_start: Union[Unset, None, datetime.date] = UNSET
    furlough_end: Union[Unset, None, datetime.date] = UNSET
    department: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        employees_full_name = self.employees_full_name
        employees_nino = self.employees_nino
        employees_payroll_reference_number = self.employees_payroll_reference_number
        furloughed_days = self.furloughed_days
        days_in_period = self.days_in_period
        hours_normally_worked = self.hours_normally_worked
        hours_furloughed = self.hours_furloughed
        percentage_of_furloughed_days_worked = self.percentage_of_furloughed_days_worked
        gross_pay = self.gross_pay
        gross_pay_claim = self.gross_pay_claim
        ni_claim = self.ni_claim
        pension_claim = self.pension_claim
        total_claim = self.total_claim
        furlough_start: Union[Unset, None, str] = UNSET
        if not isinstance(self.furlough_start, Unset):
            furlough_start = (
                self.furlough_start.isoformat() if self.furlough_start else None
            )

        furlough_end: Union[Unset, None, str] = UNSET
        if not isinstance(self.furlough_end, Unset):
            furlough_end = self.furlough_end.isoformat() if self.furlough_end else None

        department = self.department

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee is not UNSET:
            field_dict["employee"] = employee
        if employees_full_name is not UNSET:
            field_dict["employeesFullName"] = employees_full_name
        if employees_nino is not UNSET:
            field_dict["employeesNINO"] = employees_nino
        if employees_payroll_reference_number is not UNSET:
            field_dict[
                "employeesPayrollReferenceNumber"
            ] = employees_payroll_reference_number
        if furloughed_days is not UNSET:
            field_dict["furloughedDays"] = furloughed_days
        if days_in_period is not UNSET:
            field_dict["daysInPeriod"] = days_in_period
        if hours_normally_worked is not UNSET:
            field_dict["hoursNormallyWorked"] = hours_normally_worked
        if hours_furloughed is not UNSET:
            field_dict["hoursFurloughed"] = hours_furloughed
        if percentage_of_furloughed_days_worked is not UNSET:
            field_dict[
                "percentageOfFurloughedDaysWorked"
            ] = percentage_of_furloughed_days_worked
        if gross_pay is not UNSET:
            field_dict["grossPay"] = gross_pay
        if gross_pay_claim is not UNSET:
            field_dict["grossPayClaim"] = gross_pay_claim
        if ni_claim is not UNSET:
            field_dict["niClaim"] = ni_claim
        if pension_claim is not UNSET:
            field_dict["pensionClaim"] = pension_claim
        if total_claim is not UNSET:
            field_dict["totalClaim"] = total_claim
        if furlough_start is not UNSET:
            field_dict["furloughStart"] = furlough_start
        if furlough_end is not UNSET:
            field_dict["furloughEnd"] = furlough_end
        if department is not UNSET:
            field_dict["department"] = department

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        employees_full_name = d.pop("employeesFullName", UNSET)

        employees_nino = d.pop("employeesNINO", UNSET)

        employees_payroll_reference_number = d.pop(
            "employeesPayrollReferenceNumber", UNSET
        )

        furloughed_days = d.pop("furloughedDays", UNSET)

        days_in_period = d.pop("daysInPeriod", UNSET)

        hours_normally_worked = d.pop("hoursNormallyWorked", UNSET)

        hours_furloughed = d.pop("hoursFurloughed", UNSET)

        percentage_of_furloughed_days_worked = d.pop(
            "percentageOfFurloughedDaysWorked", UNSET
        )

        gross_pay = d.pop("grossPay", UNSET)

        gross_pay_claim = d.pop("grossPayClaim", UNSET)

        ni_claim = d.pop("niClaim", UNSET)

        pension_claim = d.pop("pensionClaim", UNSET)

        total_claim = d.pop("totalClaim", UNSET)

        _furlough_start = d.pop("furloughStart", UNSET)
        furlough_start: Union[Unset, None, datetime.date]
        if _furlough_start is None:
            furlough_start = None
        elif isinstance(_furlough_start, Unset):
            furlough_start = UNSET
        else:
            furlough_start = isoparse(_furlough_start).date()

        _furlough_end = d.pop("furloughEnd", UNSET)
        furlough_end: Union[Unset, None, datetime.date]
        if _furlough_end is None:
            furlough_end = None
        elif isinstance(_furlough_end, Unset):
            furlough_end = UNSET
        else:
            furlough_end = isoparse(_furlough_end).date()

        department = d.pop("department", UNSET)

        furlough_report_line = cls(
            employee=employee,
            employees_full_name=employees_full_name,
            employees_nino=employees_nino,
            employees_payroll_reference_number=employees_payroll_reference_number,
            furloughed_days=furloughed_days,
            days_in_period=days_in_period,
            hours_normally_worked=hours_normally_worked,
            hours_furloughed=hours_furloughed,
            percentage_of_furloughed_days_worked=percentage_of_furloughed_days_worked,
            gross_pay=gross_pay,
            gross_pay_claim=gross_pay_claim,
            ni_claim=ni_claim,
            pension_claim=pension_claim,
            total_claim=total_claim,
            furlough_start=furlough_start,
            furlough_end=furlough_end,
            department=department,
        )

        return furlough_report_line



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.finalise_pay_run_pay_run_json_body import FinalisePayRunPayRunJsonBody
from ...models.pay_periods import PayPeriods
from ...models.tax_year import TaxYear
from ...types import UNSET, Response, Unset


def _get_kwargs(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: FinalisePayRunPayRunJsonBody,
    ordinal: Union[Unset, None, int] = 1,
) -> Dict[str, Any]:
    url = "{}/employers/{employerId}/payrun/{taxYear}/{payPeriod}/{periodNumber}/finalise".format(
        client.base_url,
        employerId=employer_id,
        taxYear=tax_year,
        payPeriod=pay_period,
        periodNumber=period_number,
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["ordinal"] = ordinal

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[bool]:
    if response.status_code == 200:
        response_200 = cast(bool, response.json())
        return response_200
    return None


def _build_response(*, response: httpx.Response) -> Response[bool]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: FinalisePayRunPayRunJsonBody,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[bool]:
    """Finalise a PayRun (deprecated)

     This endpoint is now deprecated and will be removed in Jan 2022.
    You should instead use the Update method and set the State to Finalised.

    Returns True if the resulting FPS has been automatically submitted due to the Employers
    RTISubmissionSettings, otherwise returns false.
    You can suppress the emailing of payslips to employees by adding a key named
    dontEmailEmployeePayslips with a value of true to the body. See the related guides for more
    information.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (FinalisePayRunPayRunJsonBody):

    Returns:
        Response[bool]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: FinalisePayRunPayRunJsonBody,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[bool]:
    """Finalise a PayRun (deprecated)

     This endpoint is now deprecated and will be removed in Jan 2022.
    You should instead use the Update method and set the State to Finalised.

    Returns True if the resulting FPS has been automatically submitted due to the Employers
    RTISubmissionSettings, otherwise returns false.
    You can suppress the emailing of payslips to employees by adding a key named
    dontEmailEmployeePayslips with a value of true to the body. See the related guides for more
    information.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (FinalisePayRunPayRunJsonBody):

    Returns:
        Response[bool]
    """

    return sync_detailed(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    ).parsed


async def asyncio_detailed(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: FinalisePayRunPayRunJsonBody,
    ordinal: Union[Unset, None, int] = 1,
) -> Response[bool]:
    """Finalise a PayRun (deprecated)

     This endpoint is now deprecated and will be removed in Jan 2022.
    You should instead use the Update method and set the State to Finalised.

    Returns True if the resulting FPS has been automatically submitted due to the Employers
    RTISubmissionSettings, otherwise returns false.
    You can suppress the emailing of payslips to employees by adding a key named
    dontEmailEmployeePayslips with a value of true to the body. See the related guides for more
    information.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (FinalisePayRunPayRunJsonBody):

    Returns:
        Response[bool]
    """

    kwargs = _get_kwargs(
        employer_id=employer_id,
        tax_year=tax_year,
        pay_period=pay_period,
        period_number=period_number,
        client=client,
        json_body=json_body,
        ordinal=ordinal,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    employer_id: str,
    tax_year: TaxYear,
    pay_period: PayPeriods,
    period_number: int,
    *,
    client: Client,
    json_body: FinalisePayRunPayRunJsonBody,
    ordinal: Union[Unset, None, int] = 1,
) -> Optional[bool]:
    """Finalise a PayRun (deprecated)

     This endpoint is now deprecated and will be removed in Jan 2022.
    You should instead use the Update method and set the State to Finalised.

    Returns True if the resulting FPS has been automatically submitted due to the Employers
    RTISubmissionSettings, otherwise returns false.
    You can suppress the emailing of payslips to employees by adding a key named
    dontEmailEmployeePayslips with a value of true to the body. See the related guides for more
    information.

    Args:
        employer_id (str):
        tax_year (TaxYear):
        pay_period (PayPeriods):
        period_number (int):
        ordinal (Union[Unset, None, int]):  Default: 1.
        json_body (FinalisePayRunPayRunJsonBody):

    Returns:
        Response[bool]
    """

    return (
        await asyncio_detailed(
            employer_id=employer_id,
            tax_year=tax_year,
            pay_period=pay_period,
            period_number=period_number,
            client=client,
            json_body=json_body,
            ordinal=ordinal,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.item import Item
from ..types import UNSET, Unset

T = TypeVar("T", bound="TaxCodeChangeValues")


@attr.s(auto_attribs=True)
class TaxCodeChangeValues:
    """
    Attributes:
        employee_id (Union[Unset, int]):
        employee (Union[Unset, Item]):
        payroll_code (Union[Unset, None, str]):
        first_name (Union[Unset, None, str]):
        last_name (Union[Unset, None, str]):
        ni_number (Union[Unset, None, str]):
        period_change (Union[Unset, int]):
        previous_tax_code (Union[Unset, None, str]):
        current_tax_code (Union[Unset, None, str]):
    """

    employee_id: Union[Unset, int] = UNSET
    employee: Union[Unset, Item] = UNSET
    payroll_code: Union[Unset, None, str] = UNSET
    first_name: Union[Unset, None, str] = UNSET
    last_name: Union[Unset, None, str] = UNSET
    ni_number: Union[Unset, None, str] = UNSET
    period_change: Union[Unset, int] = UNSET
    previous_tax_code: Union[Unset, None, str] = UNSET
    current_tax_code: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        employee_id = self.employee_id
        employee: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.employee, Unset):
            employee = self.employee.to_dict()

        payroll_code = self.payroll_code
        first_name = self.first_name
        last_name = self.last_name
        ni_number = self.ni_number
        period_change = self.period_change
        previous_tax_code = self.previous_tax_code
        current_tax_code = self.current_tax_code

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if employee_id is not UNSET:
            field_dict["employeeId"] = employee_id
        if employee is not UNSET:
            field_dict["employee"] = employee
        if payroll_code is not UNSET:
            field_dict["payrollCode"] = payroll_code
        if first_name is not UNSET:
            field_dict["firstName"] = first_name
        if last_name is not UNSET:
            field_dict["lastName"] = last_name
        if ni_number is not UNSET:
            field_dict["niNumber"] = ni_number
        if period_change is not UNSET:
            field_dict["periodChange"] = period_change
        if previous_tax_code is not UNSET:
            field_dict["previousTaxCode"] = previous_tax_code
        if current_tax_code is not UNSET:
            field_dict["currentTaxCode"] = current_tax_code

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        employee_id = d.pop("employeeId", UNSET)

        _employee = d.pop("employee", UNSET)
        employee: Union[Unset, Item]
        if isinstance(_employee, Unset):
            employee = UNSET
        else:
            employee = Item.from_dict(_employee)

        payroll_code = d.pop("payrollCode", UNSET)

        first_name = d.pop("firstName", UNSET)

        last_name = d.pop("lastName", UNSET)

        ni_number = d.pop("niNumber", UNSET)

        period_change = d.pop("periodChange", UNSET)

        previous_tax_code = d.pop("previousTaxCode", UNSET)

        current_tax_code = d.pop("currentTaxCode", UNSET)

        tax_code_change_values = cls(
            employee_id=employee_id,
            employee=employee,
            payroll_code=payroll_code,
            first_name=first_name,
            last_name=last_name,
            ni_number=ni_number,
            period_change=period_change,
            previous_tax_code=previous_tax_code,
            current_tax_code=current_tax_code,
        )

        return tax_code_change_values



from enum import Enum


class UserCategory(str, Enum):
    ACTIVETRIALIST = "ActiveTrialist"
    AGEDACTIVETRIALIST = "AgedActiveTrialist"
    LAPSEDTRIALIST = "LapsedTrialist"
    ACTIVECUSTOMER = "ActiveCustomer"
    LAPSEDCUSTOMER = "LapsedCustomer"
    SUBUSER = "SubUser"
    AGEDACTIVETRIALISTANDSUBUSER = "AgedActiveTrialistAndSubUser"
    INTERNALUSER = "InternalUser"

    def __str__(self) -> str:
        return str(self.value)
